import threading
from abc import ABC, ABCMeta, abstractmethod

from setup_logger import setup_logger_devel


class AbstractThread(metaclass=ABCMeta):
    def __init__(self, daemon=True):
        """コンストラクタ
        
        Args:
            daemon (bool, optional): デーモンスレッド. Defaults to True.
        """
        self.logger = setup_logger_devel(self.__class__.__name__)
        self.__thread = None
        self.stopFlg = False
        self.isRunning = False
        self.daemon = daemon

    def start(self):
        """開始処理
        """

        self.__thread = threading.Thread(target=self.run)
        self.__thread.setDaemon(self.daemon)
        self.__thread.start()

    def run(self):
        """スレッドプロシージャ
        """
        self.logger.info("thread proc start")
        try:
            self.pre_proc()
            self.isRunning = True
            while self.stopFlg == False:
                ret = self.thread_proc()
                if ret == False:
                    self.stopFlg = True

            self.post_proc()
        except:
            self.logger.error("thread proc failed", exc_info=True)
        finally:
            self.isRunning = False
            self.logger.info("thread proc end")

    @abstractmethod
    def pre_proc(self):
        """前処理
        """
        raise NotImplementedError()

    @abstractmethod
    def thread_proc(self):
        """スレッドループ処理
        
        Returns:
            bool: 処理結果：Falseを返すとスレッド終了
        """
        return True

    @abstractmethod
    def post_proc(self):
        """後処理
        """
        raise NotImplementedError()

    def stop(self):
        """停止
        """
        self.stopFlg = True
        try:
            self.join()
        except:
            pass

        self.stopFlg = False
        self.__thread = None
        
    def restart(self):
        """再開始
        """
        self.stop()
        self.start()

    def join(self, timeout=5):
        """スレッド終了待ち
        
        Args:
            timeout (float, optional): タイムアウト時間(秒). Defaults to None.
        """
        if self.__thread is not None and self.__thread.is_alive():
            self.__thread.join(timeout)
