import json
from enum import Enum

from jnius import autoclass, cast

import common_util as util
from setup_logger import setup_logger_devel


class Action(Enum):
    REQUEST = util.PACKAGE_DOMAIN_NAME  + "." + "ecomonitor_request"
    RESPONSE = util.PACKAGE_DOMAIN_NAME + "." + "ecomonitor_response"

class Type(Enum):
    RECONNECT_REQ = "reconnect_req"
    ERASE_REQ = "erase_req"
    SYNC_REQ = "sync_req"
    CALL = "call"
    DEL = "del"
    ERASE_RES = "erase_res"
    SYNC_RES = "sync_res"
    SHOW_PROCESSING = "show_processing"
    HIDE_PROCESSING = "hide_processing"
    NOTIFY_CONN_STATUS = "notify_conn_status"
    NOTIFY_TIME_DIFF = "notify_time_diff"
    STOP_SOUND = "stop_sound"
    SOUND_CONFIG_UPDATED = "sound_config_updated"
    REQUEST_CONN_STATUS = "request_conn_status"

def send_broadcast_request(type, data):
    """ブロードキャストインテント送信(Receiver->Service)
    
    Args:
        type (string): 区分
        data (dictionary): 送信データ
    """
    logger = setup_logger_devel(__name__)
    try:
        Intent = autoclass('android.content.Intent')
        PythonActivity = autoclass('org.kivy.android.PythonActivity')
        intent = Intent(Action.REQUEST.value)
        intent.putExtra("type", type.value)
        intent.putExtra("data", json.dumps(data))
        PythonActivity.mActivity.sendBroadcast(intent)
    except:
        logger.error("send_broadcast_request failed.", exc_info=True)

def send_broadcast_response(type, data):
    """ブロードキャスト応答送信(Service->Receiver)
    
    Args:
        type (string): タイプ
        data (dict): 送信データ
    """
    logger = setup_logger_devel(__name__)
    try:
        Intent = autoclass('android.content.Intent')
        intent = Intent(Action.RESPONSE.value)
        intent.putExtra("type", type.value)
        intent.putExtra("data", json.dumps(data))
        mService = autoclass('org.kivy.android.PythonService').mService
        mService.sendBroadcast(intent)
    except:
        logger.error("send_broadcast_response failed.", exc_info=True)
