import copy
import json
import os
import pathlib
import random
from datetime import datetime

from absreceiver import AbstractReceiver
from kivy.clock import Clock


class BaseDemoReceiver(AbstractReceiver):

    """
    テスト受信機
    """
    def __init__(self, received, erased, synced, pairing, received_on_pairing, long_pressed_on_pairing, processing, shutdown, eraser_pushed, **kwargs):
        """コンストラクタ

        Args:
            received (function): 呼出受信コールバック
            erased (function): 取消受信コールバック
            synced (function): 同期コールバック
            pairing (function): ペアリングボタン押下コールバック
            received_on_pairing (function): 呼出受信コールバック(ペアリングモード時)
            long_pressed_on_pairing (function): クリア長押しコールバック
            processing (function): 処理中表示コールバック
            shutdown (function): シャットダウンコールバック
            eraser_pushed (function): 消去機押下コールバック
        """
        super(BaseDemoReceiver, self).__init__(received, erased, synced, pairing, received_on_pairing, long_pressed_on_pairing, processing, shutdown,eraser_pushed, **kwargs)
        self.recvCb = received
        self.eraseCb = erased
        self.interval_event = None

    def start(self):
        """
        開始
        """
        self.readdata = self.readJson() 
        self.recvData = []
        self.eraseData = []


        if self.interval_event is None:
            self.interval_event = Clock.schedule_interval(lambda dt: self.recv_func(), 5)
        else:
            self.interval_event()

    def requestErase(self, data, eraseFinishedCb):
        success = False

        for rd in self.recvData:
            if (('address', data['address'] ) in rd.items() 
                and ('modelNo', data['modelNo'] ) in rd.items() 
                and ('buttonNo', data['buttonNo'] ) in rd.items() 
                and ('groupId', data['groupId'] ) in rd.items() ):
                self.recvData.remove(rd)
                success = True
                break

        _data = copy.copy(data)
        _data['displayName'] = '自分で消した'
        def __async_proc():
            self.eraseCb([_data])
            eraseFinishedCb(success, _data)
        Clock.schedule_once(lambda dt: __async_proc(), 3)


    def requestSync(self, syncFinishedCb):
        pass

    def recv_func(self):
        datalist = []
        if random.randint(0,1) % 2 == 0:
            now = datetime.now()
            data = copy.copy(random.choice(self.readdata))
            data['ins_dt'] = "{0:%Y-%m-%d %H:%M:%S.%f}".format(now)
            data['upd_dt'] = "{0:%Y-%m-%d %H:%M:%S.%f}".format(now)

            datalist.append(data)
            self.recvCb(datalist)
            self.recvData.append(data)
        else:
            data = copy.copy(random.choice(self.readdata))

            for rd in self.recvData:
                if (('address', data['address'] ) in rd.items() 
                    and ('modelNo', data['modelNo'] ) in rd.items() 
                    and ('buttonNo', data['buttonNo'] ) in rd.items() 
                    and ('groupId', data['groupId'] ) in rd.items() ):
                    datalist.append(data)
                    self.recvData.remove(rd)
                    self.eraseCb(datalist)
                    break

    def stop(self):
        """
        終了
        """
        if self.interval_event is not None:
            self.interval_event.cancel()

    def pause(self):
        pass

    def resume(self):
        pass


    def readJson(self):
        """
        テストデータ読み込み
        return テストデータ（リスト）
        """
        fn = pathlib.Path('./demodata.json').resolve()
        data = []
        if os.path.exists(fn):
            file = open(fn, 'r')
            data = json.load(file)
            file.close()

        return data
