import locale
from kivy.utils import platform

PACKAGE_NAME = "ecomonitor"
PACKAGE_DOMAIN = "jp.mscreative"
PACKAGE_DOMAIN_NAME = PACKAGE_DOMAIN + "." + PACKAGE_NAME

def is_receiver_device():
    """受信機判定

    Returns:
        bool: True=受信機、False=表示機
    """
    return platform == "linux"

def is_display_device():
    """表示機判定

    Returns:
        bool: True=表示機、False=受信機
    """
    return platform != "linux"

def is_android_device():
    """Android機判定

    Returns:
        bool: True=Android機、False=Android機ではない
    """
    return platform == "android"


def get_system_language():
    """システムの言語コードをロケールから取得する。

    Returns:
        string: 言語コード
    """
    ret = "ja"
    if is_android_device():
        from jnius import autoclass
        Locale = autoclass('java.util.Locale')
        l = Locale.getDefault()
        ret = l.getLanguage()

    else:
        l = locale.getlocale()
        if l[0] is not None:
            ll = l[0].split("_")
            if len(ll) > 0:
                ret = ll[0]
    return ret

def get_system_country():
    """システムの国コードをロケールから取得する。

    Returns:
        string: 国コード
    """
    ret = "JP"
    if is_android_device():
        from jnius import autoclass
        Locale = autoclass('java.util.Locale')
        l = Locale.getDefault()
        ret = l.getCountry()

    else:
        l = locale.getlocale()
        if l[0] is not None:
            ll = l[0].split("_")
            if len(ll) > 1:
                ret = ll[1]
    return ret