from socket import AF_INET, SO_BROADCAST, SOCK_DGRAM, SOL_SOCKET, socket

import netifaces
import time

from onetimepasswd import UDPOneTimePasswd
from setup_logger import setup_logger_devel
from telegram import Telegram

logger = setup_logger_devel(__name__)

def get_broadcast_addr():
    """ブロードキャストアドレス取得
    
    Returns:
        string: ブロードキャストアドレス
    """
    broadcast = None
    try:
        broadcast = netifaces.ifaddresses('wlan0')[netifaces.AF_INET][0]['broadcast']
    except:
        logger.error("get_broadcast_addr failed.", exc_info=True)

    return broadcast

def request_server_ipaddress(port = 5000,timeout = 1):
    """IPアドレス検索
    
    Args:
        port (int, optional): ポート番号. Defaults to 5000.
        timeout (int, optional): タイムアウト（秒）. Defaults to 1.
    
    Returns:
        dict: サーバアドレス
    """
    baddress = get_broadcast_addr() 
    logger.debug("broadcst address = {}".format(baddress))
    ret = None

    with socket(AF_INET, SOCK_DGRAM) as s:
        try:
            s.setsockopt(SOL_SOCKET, SO_BROADCAST, 1)
            s.settimeout(timeout)
            t = Telegram()
            # 送信
            # otp = UDPOneTimePasswd()
            # pw = otp.calc()
            pw = "478444"
            send_data = t.create_telegram_udp_reqipaddr_D(pw)
            logger.info("send data = {}.".format(send_data.hex()))
            s.sendto(send_data, (baddress, port))
            # 受信
            msg, address = s.recvfrom(1024)
            logger.info("recv data = {}.".format(msg.hex()))
            recv_data = t.parse_udp_recv_data(msg)
            if recv_data["sender"] == Telegram.SENDER_RECV and recv_data["telgNo"] == Telegram.TELEG_UDP_REQIP:
                if "macAddress" in recv_data:
                    ret = {
                        "ipAddress": address[0],
                        "macAddress": recv_data["macAddress"] 
                    }

        except:
            logger.error("request_server_ipaddress failed.", exc_info=True)

    return ret


def request_timedelta(ipaddress, port = 5000,timeout = 1):
    """IPアドレス検索
    
    Args:
        ipaddress (string): IPアドレス.
        port (int, optional): ポート番号. Defaults to 5000.
        timeout (int, optional): タイムアウト（秒）. Defaults to 1.
    
    Returns:
        float: クライアント側の遅れ時間（秒）
    """
    ret = 0.0

    with socket(AF_INET, SOCK_DGRAM) as s:
        try:
            s.settimeout(timeout)
            t = Telegram()
            # 送信
            send_data = t.create_telegram_udp_timedelta_D()
            logger.info("send data = {}.".format(send_data.hex()))
            t1 = time.time()
            s.sendto(send_data, (ipaddress, port))
            # 受信
            msg, _ = s.recvfrom(1024)
            t4 = time.time()
            logger.info("recv data = {}.".format(msg.hex()))
            recv_data = t.parse_udp_recv_data(msg)
            if recv_data["sender"] == Telegram.SENDER_RECV and recv_data["telgNo"] == Telegram.TELEG_UDP_TIMEDELTA:
                if "rtime" in recv_data and "stime" in recv_data:
                    t2 = recv_data["rtime"]
                    t3 = recv_data["stime"]
                    ret = (t2 + t3)/2 - (t1+ t4)/2

        except:
            logger.error("request_timedelta failed.", exc_info=True)

    return ret

if __name__ == "__main__":
    # while True:
    ret = request_server_ipaddress()
    print(ret)
