window.onload = funcOnload;

/**
 * ボディオンロードイベント処理
 *
 */
function funcOnload() {
	getTerminalInfo()
	.then((termInfo) => {
		setDictionaryData("headerTerminalInfo", termInfo);
		setDictionaryData("terminalInfo", termInfo);
	});

	getNetworkInfo()
	.then((networkInfo) => {
		let USE=getResources("admin:USE");
		let NOT_USE=getResources("admin:NOT_USE");
		networkInfo.wlan.enabled = networkInfo.wlan.enabled? USE:NOT_USE;
		if (networkInfo.wlan.autoIp !== undefined) {
			networkInfo.wlan.autoIp = networkInfo.wlan.autoIp? USE:NOT_USE;
		}
		if (networkInfo.wlan.autoDNS !== undefined) {
			networkInfo.wlan.autoDNS = networkInfo.wlan.autoDNS? USE:NOT_USE;
		}
		networkInfo.eth.enabled = networkInfo.eth.enabled? USE:NOT_USE;
		if (networkInfo.eth.autoIp !== undefined) {
			networkInfo.eth.autoIp = networkInfo.eth.autoIp? USE:NOT_USE;
		}
		if (networkInfo.eth.autoDNS !== undefined) {
			networkInfo.eth.autoDNS = networkInfo.eth.autoDNS? USE:NOT_USE;
		}
		setDictionaryData("wlanNetworkInfo", networkInfo.wlan);
		setDictionaryData("ethNetworkInfo", networkInfo.eth);
	});

	let groupList;
	getGroupData()
	.then((groups) => {
		groupList = groups;

		const MAX_GROUP_NUM = 3;
		let dispGroups = JSON.parse(JSON.stringify(groups)); // clone
		if (dispGroups.length > MAX_GROUP_NUM) {
			dispGroups.length = MAX_GROUP_NUM;
		}
		createGroupTableRow(dispGroups);
		setDictionaryDataToTable("groupTbody", dispGroups);


		let remain = document.getElementById("remainGroupNum");
		remain.textContent = ""
		if (groups.length - dispGroups.length) {
			// remain.textContent ="その他 " + (groups.length - dispGroups.length) + "グループ登録済み";
			remain.textContent = formatMessage(getResources("admin:GROUP_REMAIN_COUNT"), (groups.length - dispGroups.length));
		}

		return getCallerData(true)
	})
	.then((callerData) => {
		let callers = callerData["callers"];
		let sounds = callerData["sounds"];
		const MAX_CALLER_NUM = 10;

		let dispCallers = JSON.parse(JSON.stringify(callers)); // clone
		if (dispCallers.length > MAX_CALLER_NUM) {
			dispCallers.length = MAX_CALLER_NUM;
		}

		createCallerTableRow(dispCallers);

		let convData = convertCallerData(dispCallers, groupList, sounds);
		setDictionaryDataToTable("callerTbody", convData);
		initColorBox("callerTbody", convData);

		let remain = document.getElementById("remainCallerNum");
		remain.textContent = ""
		if (callers.length - dispCallers.length) {
			// remain.textContent ="その他 " + (callers.length - dispCallers.length) + "個登録済み";
			remain.textContent = formatMessage(getResources("admin:CALLER_REMAIN_COUNT"), (callers.length - dispCallers.length));
		}
	});

	getEraserData()
	.then((eraserData) => {
		let erasers = eraserData["erasers"];
		createEraserTableRow(erasers);
		setDictionaryDataToTable("eraserTbody", erasers);
	});

	getOtherInfo()
	.then((otherInfo) => {
		setDictionaryData("sound", otherInfo.sound);
		if (otherInfo.timeSetting.ntp) {
			otherInfo.timeSetting["method"] = getResources("admin:SET_TIME_METHOD_NTP");
		}
		else {
			otherInfo.timeSetting["method"] = getResources("admin:SET_TIME_METHOD_MANUAL");
		}
		otherInfo.timeSetting.interval = convertInterval(otherInfo.timeSetting.interval);
		setDictionaryData("timeSet", otherInfo.timeSetting);
		// otherInfo.logServer.interval = convertInterval(otherInfo.logServer.interval);
		// setDictionaryData("logServer", otherInfo.logServer);
	});

}

/**
 *　タブ切り替える時の処理
 *
 * @param {string} view - 切り替えページ
 */
function funcTabChange(view) {
	// var f = document.forms[0];
	// f.action = view;
	// f.submit();
    window.location.href = view;
}


/**
 * グループ表示テーブルの行作成
 *
 * @param {Array} groups - グループ情報
 */
function createGroupTableRow(groups) {
	let groupTbody = document.querySelector("#groupTbody");
	for(let i = 1; i <= groups.length; i++) {
		let tr = document.createElement("tr");
		tr.setAttribute("data-row", i);
		tr.classList.add("row-space");
		["groupId", "name"].forEach((dataName) => {
			let td = document.createElement("td");
			if(dataName=="groupId") {
				td.setAttribute("data-label", getResources("admin:data-label:groupId"));
			}
			else if(dataName=="name") {
				td.setAttribute("data-label", getResources("admin:data-label:groupName"));
			}
			td.setAttribute("data-name", dataName);
			tr.appendChild(td);
		});
		groupTbody.appendChild(tr);
	}
}

/**
 * 呼出機表示テーブルの行作成
 *
 * @param {Array} callers - 呼出機情報
 */
function createCallerTableRow(callers) {
	let callerTbody = document.querySelector("#callerTbody");
	let row = 1;
	let prevAddress = "";
	callers.forEach((caller) => {
		let address = caller["address"];
		let firstTr = false;
		let buttonLen = 0;
		if (address != prevAddress) {
			buttonLen = callers.filter((c) => c.address == address).length;
			firstTr = true;
		}
		let tr = document.createElement("tr");
		tr.setAttribute("data-row", row);
		if (firstTr == true) {
			tr.classList.add("row-space");
			let tdAddress = document.createElement("td");
			tdAddress.setAttribute("rowspan", buttonLen);
			tdAddress.setAttribute("data-label", getResources("admin:data-label:address"));
			["address", "modelNo"].forEach((dataName) => {
				let p = document.createElement("p");
				p.setAttribute("data-name", dataName);
				if (dataName == "modelNo") {
					p.classList.add("bracket");
				}
				tdAddress.appendChild(p);
			});
			tr.appendChild(tdAddress);
		}

		["buttonNo", "line", "call", "color", "sound", "priority"].forEach((dataName) => {
			let td = document.createElement("td");
			if (dataName == "buttonNo") {
				td.setAttribute("data-label", getResources("admin:data-label:buttonNo"));
			}
			else if (dataName == "line") {
				["line1", "line2"].forEach((dataName1) => {
					let span = document.createElement("span");
					span.setAttribute("data-name", dataName1);
					td.appendChild(span);
					let br = document.createElement('br');
					td.appendChild(br);
				});
			}
			else if (dataName == "call") {
				td.setAttribute("data-label", getResources("admin:data-label:callGroup"));
			}
			else if (dataName == "color") {
				td.setAttribute("data-label", getResources("admin:data-label:color"));
				td.style.whiteSpace = "nowrap";
			}
			else if (dataName == "sound") {
				td.setAttribute("data-label", getResources("admin:data-label:sound"));
			}
			else if (dataName == "priority") {
				td.setAttribute("data-label", getResources("admin:data-label:priority"));
			}
			td.setAttribute("data-name", dataName);
			tr.appendChild(td);
		});

		callerTbody.appendChild(tr);
		prevAddress = address;
		row++;

	});
}

/**
 * 呼出機情報のデータを画面表示用のデータに変換する
 *
 * @param {Array} callerList　- 呼出機情報
 * @param {Array} groups - グループ情報
 * @param {object} sounds - 音情報
 * @returns 呼出機情報（画面表示用）
 */
function convertCallerData(callerList, groups, sounds) {
	let retList = [];
	let lang = getLanguage();
	callerList.forEach((caller) => {
		let data = {};
		for (let name in caller) {
			if(name == "call") {
				if (caller[name] === "") {
					data[name] = "";
				}
				else if (caller[name] == 0) {
					data[name] = getResources("admin:ALL_DISP_DEVICE");
				}
				else if (caller[name] == 100) {
					data[name] = getResources("admin:CANCEL");
				}
				else if (caller[name] == 101) {
					data[name] = getResources("admin:INVALID");
				}
				else {
					let g = groups.filter(x => x.groupId==caller[name]);
					if (g.length > 0) {
						data[name] = g[0].name;
					}
					else {
						data[name] = caller[name];
					}
				}
			}
			else if(name == "sound") {
				let sinfos = sounds.filter(s => s.no == caller[name])
				if (sinfos.length > 0) {
					data[name] = sinfos[0].name[lang];
				}
				else {
					data[name] = caller[name];
				}
			}
			else if(name == "priority") {
				if (caller["call"] != 100 && caller["call"] != 101) {
					if (caller["priority"] == 0) {
						data[name] = getResources("admin:PRIORITY_NORMAL");
					}
					else {
						data[name] = getResources("admin:PRIORITY_HIGH");
					}
				}
				else {
					data[name] = "";
				}
			}
			else {
				data[name] = caller[name];
			}
		}
		retList.push(data);
	});
	return retList;
}

/**
 * 呼出機の色の表示を設定する
 *
 * @param {string} parentTbodyId - 親テーブルID
 * @param {Array} dataList - 呼出機情報
 */
function initColorBox(parentTbodyId, dataList) {
	let tbody = document.querySelector("#" + parentTbodyId);
	for (let i = 1; i <= dataList.length; i++) {
		let td = getDataElemFromTable(parentTbodyId, i, "color");
		switch(dataList[i-1].color) {
			case "R":
				td.textContent = getResources("admin:COLOR_RED");
				break;
			case "G":
				td.textContent = getResources("admin:COLOR_GREEN");
				break;
			case "B":
				td.textContent = getResources("admin:COLOR_BLUE");
				break;
			case "Y":
				td.textContent = getResources("admin:COLOR_YELLOW");
				break;
			case "W":
				td.textContent = getResources("admin:COLOR_WHITE");
				break;
		}
		let colorbox = document.createElement("span");
		colorbox.classList.add("color-box");
		colorbox.setAttribute("data-color", dataList[i-1].color);
		td.appendChild(colorbox);
	}
}

/**
 * 間隔を表示用に変換する
 *
 * @param {string} interval - 間隔
 * @returns 間隔（変換後）
 */
function convertInterval(interval) {
	let ret = interval;
	if (interval && interval.length == 3) {
		let unit = "";
		switch(interval.charAt(0)) {
			case "M":
				unit = getResources("admin:MINUTES");
				break;
			case "H":
				unit = getResources("admin:HOURS");
				break;
			case "D":
				unit = getResources("admin:DAYS");
				break;
			case "W":
				unit = getResources("admin:WEEKS");
				break;
			default:
				break;
		}

		ret = parseInt( interval.substr(1), 10) + unit;
	}
	return ret;
}

/**
 * 消去機表示テーブルの行作成
 *
 * @param {Array} erasers - 消去機情報
 */
 function createEraserTableRow(erasers) {
	let eraserTbody = document.querySelector("#eraserTbody");
	for(let i = 1; i <= erasers.length; i++) {
		let tr = document.createElement("tr");
		tr.setAttribute("data-row", i);
		tr.classList.add("row-space");

		let tdAddress = document.createElement("td");
		tdAddress.setAttribute("data-label", getResources("admin:data-label:address"));
		["address", "modelNo"].forEach((dataName) => {
			let p = document.createElement("p");
			p.setAttribute("data-name", dataName);
			if (dataName == "modelNo") {
				p.classList.add("bracket");
			}
			tdAddress.appendChild(p);
		});
		tr.appendChild(tdAddress);

		let td = document.createElement("td");
		td.setAttribute("data-label", getResources("admin:data-label:eraserName"));
		td.setAttribute("data-name", "name");
		tr.appendChild(td);

		eraserTbody.appendChild(tr);
	}
}
