
window.onload = funcOnload;

let groups = [];
let sounds = [];
let delCallerList = [];
let orgCalelerList = []
/**
 * ボディオンロードイベント処理
 *
 */
function funcOnload() {
	// showAdminPage();

	getTerminalInfo()
	.then((termInfo) => {
		setDictionaryData("headerTerminalInfo", termInfo);
	});

	getGroupData()
	.then((groupList) =>{
		groups = groupList;
		return getCallerData(true);
	})
	.then((callerData) => {
		let callers = callerData["callers"];
		sounds = callerData["sounds"];
		orgCalelerList = callers;
		createTableRow("tableBody", callers, groups, sounds);
		callers.forEach(c=>{
			c["defaultColor"] = c["color"];
			c["defaultSound"] = c["sound"];
			c["defaultPriority"] = c["priority"];
		});
		setDictionaryDataToTable("tableBody", callers);
		initColorBox("tableBody", callers);
		initSounds(sounds);

		addOnChangeListnerToInput("tableBody", funcChange);

		// 新規データの監視
		startWatchNewCallers(5000);

		document.getElementById('delCallerBtn').onclick = deleteCallers;
		document.getElementById('updateBtn').onclick = funcUpdate;
	});


}

/**
 * 新規呼出機データの監視
 */
/**
 * 
 * @param {int} interval - 監視間隔（ミリ秒）
 */
function startWatchNewCallers(interval) {
	setInterval(()=> {
		getCallerData(false)
		.then((callerData) => {
			// 新規データを取得
			let callers = callerData["callers"];
			// 受信機で追加された呼出機
			let newCallers = callers.filter((c) => {
				// 既存のデータに一致するか
				let l =  orgCalelerList.filter( oc => oc.address == c.address);
				if (l.length == 0) {
					// 不一致
					return true;
				}
				else {
					// 一致
					return false;
				}
			});
			// 受信機で削除された呼出機
			let deletedCallers = orgCalelerList.filter((oc) => {
				let l = callers.filter( c => c.address == oc.address);
				if (l.length == 0) {
					// 不一致
					return true;
				}
				else {
					// 一致
					return false;
				}
			});

			// 受信機で削除された呼出機を表示から削除する。
			if (deletedCallers.length > 0) {
				deleteCallersByReceiver(deletedCallers);
				orgCalelerList = orgCalelerList.filter((oc) => {
					let l = deletedCallers.filter(dc => dc.address == oc.address);
					if (l.length == 0) {
						return true;
					}
					else {
						return false;
					}
				});
			}
			// 受信機で追加された呼出機を表示する。
			if (newCallers.length > 0) {
				insertCallers(newCallers);
				orgCalelerList = orgCalelerList.concat(newCallers)
			}

		});
	}, interval);
}

/**
 * 呼出機を追加表示する。
 * @param {Array} newCallers - 新規追加された呼出機 
 */
function insertCallers(newCallers) {
	let table = createDummyTable("dummyTable");
	document.body.appendChild(table);
	let row = orgCalelerList.length;

	createTableRow("dummyTable", newCallers, groups, sounds);
	
	setDictionaryDataToTable("dummyTable", newCallers);
	initColorBox("dummyTable", newCallers);

	addOnChangeListnerToInput("dummyTable", funcChange);

	let tbody = document.querySelector("#tableBody");
	document.querySelectorAll("#dummyTable tr").forEach((tr) => {
		tr.setAttribute("data-row", row)
		tbody.appendChild(tr);
		row++;
	});

	updateDataRowNo();

	document.body.removeChild(table);
}

/**
 * 受信機で削除された呼出機をテーブルから削除する。
 * @param {Array} deletedCallers - 受信機で削除された呼出機 
 */
function deleteCallersByReceiver(deletedCallers) {
	deletedCallers.forEach((dc) => {
		trList = findTrByAddress(dc.address);
		trList.forEach((tr) => {
			tr.remove();
		});
	});

	updateDataRowNo();
}

/**
 * trのdata-row属性の値を振り直す
 */
function updateDataRowNo() {
	let tbody = document.querySelector("#tableBody");
	let cnt = 1;
	tbody.querySelectorAll("tr[data-row]").forEach((tr) => {
		tr.setAttribute("data-row", cnt);
		cnt++;
	});
}

/**
 * ダミーのテーブルを作成する
 * @param {string} tbodyId - tbodyのId
 */
function createDummyTable(tbodyId) {
	let table = document.createElement("table");
	table.style.display = "none";
	let tbody = document.createElement("tbody");
	tbody.setAttribute("id", tbodyId);
	table.appendChild(tbody);
	return table;
}

/**
 * 呼出機表示テーブルの行を生成
 * 
 * @param {string} tbodyId - テーブルのID 
 * @param {Array} callers - 呼出機 
 * @param {Array} groups - グループ
 * @param {Array} sounds - 音情報
 */
function createTableRow(tbodyId, callers, groups, sounds) {
	let tbody = document.querySelector("#" + tbodyId);

	let row = 1;
	let prevAddress = "";
	callers.forEach((caller) => {
		let address = caller["address"];
		let firstTr = false;
		let buttonLen = 0;
		if (address != prevAddress) {
			buttonLen = callers.filter((c) => c.address == address).length;
			firstTr = true;
		}
		let tr;
		if (firstTr) {
			tr = createFirstTr(buttonLen, row, caller, groups, sounds);
		}
		else {
			tr = createTr(row, caller, groups, sounds);
		}
		tbody.appendChild(tr);
		prevAddress = address;
		row++;

	})
}

/**
 * TR要素生成（呼出機の1行目）
 * 
 * @param {int} buttonLen - ボタンの数
 * @param {int} row - 行
 * @param {object} caller - 呼出機 
 * @param {Array} groups - グループ 
 * @param {Array} sounds - 音情報
 */
function createFirstTr(buttonLen, row, caller, groups, sounds) {
	let tr = document.createElement("tr");
	tr.setAttribute("data-row", row);
	tr.classList.add("row-space");

	let tdBtn = createTd();
	tdBtn.classList.add("h-row");
	tdBtn.setAttribute("rowspan", buttonLen);
	tdBtn.setAttribute("data-label", getResources("caller:data-label:delete"));
	let delBtn = document.createElement('input');
	delBtn.setAttribute("type", "checkbox");
	delBtn.onclick = (e) => {
		// 削除チェックボックスクリック
		// e.preventDefault();
		let tr = e.target.parentNode.parentNode;
		let address = tr.querySelector("*[data-name='address']");
		if (tr.classList.contains('delete-row')) {
			// 削除済みは戻す
			findTrByAddress(address.textContent).forEach((tr) => {
				tr.classList.remove('delete-row');
				tr.querySelectorAll("td:not(.disabled-cell) input,td:not(.disabled-cell) select").forEach((elem) => {
					elem.disabled = false;
				});
	
			});

			delCallerList = delCallerList.filter( a => a != address.textContent);
		}
		else {
			// 行を灰色にする
			findTrByAddress(address.textContent).forEach((tr) => {
				tr.classList.add('delete-row');
				tr.querySelectorAll("input,select").forEach((elem) => {
					if (elem != e.target) {
						elem.disabled = true;
					}
				});
			});
			// 削除リストに追加する。
			delCallerList.push(address.textContent);
		}
		document.getElementById("delCallerBtn").disabled = !hasDeleteCallers();
	}
	tdBtn.appendChild(delBtn);

	let tdAddress = createTd();
	tdAddress.setAttribute("data-label", getResources("caller:data-label:address"));
	tdAddress.setAttribute("rowspan", buttonLen);
	["address", "modelNo"].forEach((dataName) => {
		let p = document.createElement("p");
		p.setAttribute("data-name", dataName);
		if (dataName == "modelNo") {
			p.classList.add("bracket");
		}
		tdAddress.appendChild(p);
	});


	tr.appendChild(tdBtn);
	tr.appendChild(tdAddress);

	createTr(row, caller, groups, sounds).querySelectorAll("td").forEach((td) => {
		tr.appendChild(td);
	});

	return tr;
}

/**
 * TR要素生成（呼出機）
 * 
 * @param {int} row - 行
 * @param {object} caller - 呼出機 
 * @param {Array} groups - グループ 
 * @param {Array} sounds - 音情報
 * @return TR Element
 */
function createTr(row, caller, groups, sounds) {
	let tr = document.createElement("tr");
	tr.setAttribute("data-row", row);

	let tdNo = createTd("buttonNo");
	tdNo.setAttribute("data-label", getResources("caller:data-label:buttonNo"));

	let tdCallerName = document.createElement("td");
	tdCallerName.setAttribute("data-label", getResources("caller:data-label:callerName"));
	["line1", "line2"].forEach((dataName) => {
		let input;
		if (dataName=="line1") {
			input = createInput({"maxlength":20, "style": "width:160px;"}, dataName, 20);
		}
		else {
			input = createInput({"maxlength":12, "style": "width:100px;"}, dataName, 12);
		}
		if (caller["call"] == "100" || caller["call"] == "101") {
			input.disabled = true;
		}
		tdCallerName.appendChild(input);
		let br = document.createElement('br');
		tdCallerName.appendChild(br);
	});

	let opt = [];
	opt.push({ "value": "0","text": getResources("caller:ALL_DISP_DEVICE")});
	groups.forEach((g)=> {
		opt.push({
			"value": g.groupId,
			"text":g.name,
		})
	});
	if(caller["modelNo"].charAt(0) != 'A') {
		//一つボタンの場合は「取り消し」は不要
		opt.push({ "value": "100","text": getResources("caller:CANCEL")});
	}
	opt.push({ "value": "101","text": getResources("caller:INVALID")});
	let tdSelCall = createTdSelect(opt, "call");
	tdSelCall.setAttribute("data-label", getResources("caller:data-label:callGroup"));
	let tdSelColor = createTdSelect([
		{ "value": "R","text": getResources("caller:COLOR_RED")},
		{ "value": "G","text": getResources("caller:COLOR_GREEN")},
		{ "value": "B","text": getResources("caller:COLOR_BLUE")},
		{ "value": "Y","text": getResources("caller:COLOR_YELLOW")},
		{ "value": "W","text": getResources("caller:COLOR_WHITE")},
	], "color");
	tdSelColor.setAttribute("data-label", getResources("caller:data-label:color"));
	tdSelColor.style.whiteSpace = "nowrap";
	let colorSelect = tdSelColor.querySelector("select");
	colorSelect.addEventListener('change', (e) => {
		let val = e.target.value;
		let select = e.target;
		let span = select.parentNode.querySelector("span");
		span.setAttribute("data-color", val);
		if(val) {
			let hiddenColor = colorSelect.parentNode.querySelector('input[type="hidden"][data-name="defaultColor"]');
			if(hiddenColor) {
				hiddenColor.value = val;
			}
		}
	});
	let span = document.createElement("span");
	span.classList.add('color-box');
	span.setAttribute("data-size", "large");
	colorSelect.parentNode.appendChild(span);
	let hidden = createHidden("address");
	colorSelect.parentNode.appendChild(hidden);
	let hiddenDefaultColor = createHidden("defaultColor");
	colorSelect.parentNode.appendChild(hiddenDefaultColor);

	let optSound = [];
	let lang = getLanguage();
	sounds.forEach((s)=> {
		optSound.push({
			"value": s.no,
			"text": s.name[lang],
		})
	});
	let tdSelSound = createTdSelect(optSound, "sound");
	tdSelSound.setAttribute("data-label", getResources("caller:data-label:sound"));
	tdSelSound.style.whiteSpace = "nowrap";
	let soundSelect = tdSelSound.querySelector("select");
	let hiddenDefaultSound = createHidden("defaultSound");
	soundSelect.parentNode.appendChild(hiddenDefaultSound);
	soundSelect.addEventListener('change', (e) => {
		let val = e.target.value;
		if(val) {
			hiddenDefaultSound.value = val;
		}
	});

	let tdSelPriority = createTdSelect([
		{ "value": "1","text": getResources("caller:PRIORITY_HIGH")},
		{ "value": "0","text": getResources("caller:PRIORITY_NORMAL")},
	], "priority");
	tdSelPriority.setAttribute("data-label", getResources("caller:data-label:priority"));
	tdSelPriority.style.whiteSpace = "nowrap";
	let prioritySelect = tdSelPriority.querySelector("select");
	let hiddenDefaultPriority = createHidden("defaultPriority");
	prioritySelect.parentNode.appendChild(hiddenDefaultPriority);
	prioritySelect.addEventListener('change', (e) => {
		let val = e.target.value;
		if(val) {
			hiddenDefaultPriority.value = val;
		}
	});

	if (caller["call"] == "100" || caller["call"] == "101") {
		// 取消、無効
		colorSelect.disabled = true;
		soundSelect.disabled = true;
		prioritySelect.disabled = true;
		tdCallerName.classList.add('disabled-cell');
		tdSelColor.classList.add('disabled-cell');
		tdSelSound.classList.add('disabled-cell');
		tdSelPriority.classList.add('disabled-cell');
	}
	else {
		colorSelect.disabled = false;
		soundSelect.disabled = false;
		prioritySelect.disabled = false;
		tdCallerName.classList.remove('disabled-cell');
		tdSelColor.classList.remove('disabled-cell');
		tdSelSound.classList.remove('disabled-cell');
		tdSelPriority.classList.remove('disabled-cell');
	}

	let callSelect = tdSelCall.querySelector("select");
	callSelect.addEventListener('change', (e) => {
		let value = e.target.value;
		if (value == "100" || value == "101") {
			// 取消、無効
			colorSelect.disabled = true;
			soundSelect.disabled = true;
			prioritySelect.disabled = true;
			colorSelect.value = "";
			soundSelect.value = "";
			tdCallerName.classList.add('disabled-cell');
			tdSelColor.classList.add('disabled-cell');
			tdSelSound.classList.add('disabled-cell');
			tdSelPriority.classList.add('disabled-cell');
			tdCallerName.querySelectorAll('input').forEach(el => el.disabled = true);
			// prioritySelect.value = "0";
			fireEvent(colorSelect, "HTMLEvents", "change");
			fireEvent(soundSelect, "HTMLEvents", "change");
			// fireEvent(prioritySelect, "HTMLEvents", "change");
		}
		else {
			colorSelect.disabled = false;
			soundSelect.disabled = false;
			prioritySelect.disabled = false;
			colorSelect.value = hiddenDefaultColor.value?hiddenDefaultColor.value:"R";
			soundSelect.value = hiddenDefaultSound.value?hiddenDefaultSound.value:"01";
			prioritySelect.value = hiddenDefaultPriority.value?hiddenDefaultPriority.value:"0";
			tdCallerName.classList.remove('disabled-cell');
			tdSelColor.classList.remove('disabled-cell');
			tdSelSound.classList.remove('disabled-cell');
			tdSelPriority.classList.remove('disabled-cell');
			tdCallerName.querySelectorAll('input').forEach(el => el.disabled = false);

			fireEvent(colorSelect, "HTMLEvents", "change");
			fireEvent(soundSelect, "HTMLEvents", "change");
			fireEvent(prioritySelect, "HTMLEvents", "change");
		}
	});

	let playBtn = document.createElement("button");
	playBtn.classList.add("image-button");
	playBtn.style.marginLeft = "5px";
	let img = document.createElement("img");
	img.setAttribute("src", "img/play.png");
	img.style.width = "20px";
	img.style.height = "20px";
	playBtn.appendChild(img);
	playBtn.onclick = (e) => {
		let btn = e.currentTarget;
		let id = btn.parentNode.querySelector("select").value;
		playSound(id);
	};
	tdSelSound.appendChild(playBtn);

	tr.appendChild(tdNo);
	tr.appendChild(tdCallerName);
	tr.appendChild(tdSelCall);
	tr.appendChild(tdSelColor);
	tr.appendChild(tdSelSound);
	tr.appendChild(tdSelPriority);
	return tr;
}

/**
 * TD Element生成
 * @param {string} dataName - データ名
 * @returns TD Element
 */
function createTd(dataName) {
	let td = document.createElement("td");
	td.classList.add('inputs');
	if(dataName) {
		td.setAttribute("data-name", dataName);
	}

	return td;
}

/**
 * Input Element生成
 * @param {object} attrs - 属性
 * @param {string}} dataName - データ名
 * @return Input Element
 */
function createInput(attrs, dataName, lenbyte) {
	let input = document.createElement('input');
	input.setAttribute("type", "text");
	input.classList.add('gnp-input-text');
	// input.classList.add('input-s');
	// input.style.width = "100%";
	if(dataName) {
		input.setAttribute("data-name", dataName);
	}
	for (let  key in attrs) {
		input.setAttribute(key, attrs[key]);
	}
	if(lenbyte) {
		input.addEventListener('change', (ev) => {
			ev.preventDefault();
			let lb = lenb(input.value);
			if(lb > lenbyte) {
				input.value = leftb(input.value, lenbyte);
			}
		});
	}
	return input;
}

/**
 * TD内にInput Element生成
 * @param {object} attrs - 属性
 * @param {string}} dataName - データ名
 * @return TD Element
 */
function createTdInput(attrs, dataName) {
	let td = createTd();
	let input = createInput(attrs);
	if(dataName) {
		input.setAttribute("data-name", dataName);
	}
	td.appendChild(input);
	return td;
}

/**
 * TD内にselect Element生成
 * @param {Array} optlist - オプション 
 * @param {string} dataName - データ名
 * @return TD Element
 */
function createTdSelect(optlist, dataName) {
	let td = createTd();
	let select = document.createElement('select');
	select.classList.add('gnp-list-inline');
	if(dataName) {
		select.setAttribute("data-name", dataName);
	}

	optlist.forEach((opt) => {
		let option = document.createElement('option');
		for(let atr in opt) {
			if (atr == "text") {
				option.textContent = opt[atr];
			}
			else {
				option.setAttribute(atr, opt[atr]);
			}
		}
		select.appendChild(option);
	});

	td.appendChild(select);

	return td;
}
/**
 * input(type=hidden) Element生成
 * @param {string} dataName - データ名
 * @return input Element
 */
function createHidden(dataName) {
	let input = document.createElement('input');
	input.setAttribute("type", "hidden");
	if(dataName) {
		input.setAttribute("data-name", dataName);
	}
	return input;
}

/**
 * アドレスが一致するTRを探す
 * @param {string} address - アドレス
 * @returns TRのList
 */
function findTrByAddress(address) {
	let trList = [];
	document.querySelectorAll('input[type="hidden"][data-name="address"][value="'+ address +'"]').forEach((elem) => {
		trList.push(elem.parentNode.parentNode);
	});
	return trList;
}


/**
 * 呼出機情報のデータを画面表示用のデータに変換する
 * @param {Array} callerList - 呼出機データ
 * @returns 画面表示用データ
 */
function convertCallerData(callerList) {
	let retList = [];
	callerList.forEach((caller) => {
		let data = {};
		for (let name in caller) {
			if (name == "buttons") {
				let buttons = caller.buttons;
				data["buttonLen"] = buttons.length;
				let i = 0;
				buttons.forEach((button) => {
					data = JSON.parse(JSON.stringify(data)); //clone
					for(let name in button) {
						if (name == "no") {
							data["buttonNo"] = button[name];
						}
						else {
							data[name] = button[name];
						}
					}
					retList.push(data);
					i++;
				});
			}
			else if (name == "no") {
				data["modelNo"] = caller[name];
			}
			else{
				data[name] = caller[name];
			}
		}
	});
	return retList;
}

/**
 * 呼出機の色の表示を設定する
 *
 * @param {string} parentTbodyId - 親テーブルID
 * @param {Array} dataList - 呼出機情報
 */
function initColorBox(parentTbodyId, dataList) {
	let tbody = document.querySelector("#" + parentTbodyId);
	for (let i = 1; i <= dataList.length; i++) {
		let colorBox = tbody.querySelector("tr[data-row='"+ i +"'] span.color-box");
		if(colorBox) {
			colorBox.setAttribute("data-color", dataList[i - 1].color);
		}
	}
}

/**
 * 音の初期化（audioタグ生成）
 * @param {Array} sounds -　音情報
 */
function initSounds(sounds) {
	let div = document.getElementById("audioDiv");
	sounds.forEach((s) => {
		let audio = document.createElement("audio");
		audio.setAttribute("data-no", s.no);
		audio.setAttribute("preload", "auto");
		let source = document.createElement("source");
		source.setAttribute("src", s.path);
		source.setAttribute("type", s.type);
		audio.appendChild(source);

		div.appendChild(audio);
	});
}

/**
 * 音を鳴らす
 * @param {string} no - 音No.
 */
function playSound(no) {
	let audio = document.querySelector("audio[data-no='"+ no +"']");
	if(audio) {
		audio.play();
	}
}

/**
 * 入力項目チェック
 * @returns 結果
 */
function funcInputCheck() {
	let bCheckResult = true;

	// 何かやることがあれば・・・

	if(bCheckResult == false) {
		showDivError();
		return false;
	} else {
		hideDivError();
		return true;
	}
}

/**
 * 入力コントロールの無効・有効
 * @param {boolean} disabled - 無効
 */
function disableControls(disabled) {
	setInputDisabled("tableBody", disabled);
	document.getElementById("delCallerBtn").disabled = disabled;
	document.getElementById("updateBtn").disabled = disabled;
}

/**
 * 更新実行
 */
function doUpdate() {
	let updateData = getDataListFromTable("tableBody");

	let param = {
		"delete": delCallerList,
		"update": updateData
	};
	console.log(param);

	disableControls(true);

	doPost("/update/callers", param, handleLoginError)
	.then( (resp) => {
		if (resp.status == "ok") {
			location.reload();
		}
		else {
			showServerError(getResources("caller:UPDATE_FAILED"));
			disableControls(false);
		}
	})
	.catch( () => {
		showServerError(getResources("caller:UPDATE_FAILED"));
		disableControls(false);
	});
}

/**
 * 削除チェックをいれた呼出機が存在するか
 * @return true:存在する、false:存在しない。
 */
function hasDeleteCallers() {
	let parentTable = document.getElementById("tableBody");
	let trList = parentTable.querySelectorAll("tr.delete-row[data-row]");
	if (trList.length == 0) {
		return false;
	}
	return true;
}

/**
 * 呼出機削除
 */
function deleteCallers() {
	if (!hasDeleteCallers()) {
		return;
	}
	if (!confirm(getResources("caller:DELETE_CONFIRM"))) {
		return;
	}

	let param = {
		"delete": delCallerList,
	};

	console.log(param);
	disableControls(true);
	doPost("/delete/callers", param, handleLoginError)
	.then((resp) => {
		if (resp.status == "ok") {
			let parentTable = document.querySelector("#tableBody");
			let trList = parentTable.querySelectorAll("tr.delete-row[data-row]");
			trList.forEach(tr => {
				tr.remove();
			});
			delCallerList.forEach(address =>{
				orgCalelerList = orgCalelerList.filter(oc => oc.address != address)
			});
			delCallerList.length = 0;
			disableControls(false);
			document.getElementById("delCallerBtn").disabled = !hasDeleteCallers();
		}
		else {
			showServerError(getResources("caller:DELETE_FAILED"));
			disableControls(false);
		}
	})
	.catch(() => {
		showServerError(getResources("caller:DELETE_FAILED"));
		disableControls(false);
	});
}
