"use strict";  //Use the function form of 'use strict'.

/**
 * データをHTMLに反映する
 *
 * @param {string} parentId - 親Id
 * @param {object} data - データ
 */
function setDictionaryData(parentId, data) {
	let parentElem = document.querySelector("#" + parentId);
	for (let name in data) {
		let elms = parentElem.querySelectorAll("*[data-name='"+ name+"']");
		elms.forEach(function(elm) {
			if(["select", "input"].indexOf(elm.tagName.toLowerCase()) >= 0) {
				elm.value = data[name];
			}
			else {
				elm.textContent = data[name];
			}

		})
	}
}

/**
 * HTMLからデータを保持しているElementを取得する
 *
 * @param {string} parentId - 親ID
 * @param {string} dataName - データ名
 * @returns データElement
 */
function getDataElem(parentId, dataName) {
	let parentElem = document.querySelector("#" + parentId);
	return parentElem.querySelector("*[data-name='"+ dataName +"']");
}

/**
 * HTMLからデータを取得する
 *
 * @param {string} parentId - 親ID
 * @returns データ
 */
function getDictionaryData(parentId) {
	let parentElem = document.querySelector("#" + parentId);
	var data = {};
	let elems = parentElem.querySelectorAll("*[data-name]");
	elems.forEach(function(elem) {
		let name = elem.getAttribute("data-name");
		let value;
		if(["select", "input"].indexOf(elem.tagName.toLowerCase()) >= 0) {
			value = elem.value;
		}
		else {
			value = elem.textContent;
		}
		data[name] = value;
	});
	return data;
}

/**
 * データをHTMLに反映する
 *
 * @param {string} parentTbodyId - 親テーブルID
 * @param {Array} dataList - データリスト
 */
function setDictionaryDataToTable(parentTbodyId, dataList) {
	let parentTable = document.querySelector("#" + parentTbodyId);
	for (let i = 0; i < dataList.length; i++) {
		let elmTr = parentTable.querySelector("tr[data-row='"+ (i + 1) +"']"); 
		for (let name in dataList[i]) {
			let elms = elmTr.querySelectorAll("*[data-name='"+ name+"']");
			elms.forEach(function(elm) {
				if(["select", "input"].indexOf(elm.tagName.toLowerCase()) >= 0) {
					elm.value = dataList[i][name];
				}
				else {
					elm.textContent = dataList[i][name];
				}
	
			});
		}	

	}
}

/**
 * HTMLからデータを保持しているElementを取得する
 *
 * @param {string} parentTbodyId - 親テーブルID
 * @param {int} row - 行番号
 * @param {string}} dataName - データ名
 * @returns データElement
 */
function getDataElemFromTable(parentTbodyId, row, dataName) {
	let parentTable = document.querySelector("#" + parentTbodyId);
	return parentTable.querySelector("tr[data-row='"+ row +"'] *[data-name='" + dataName+ "']");
}

/**
 * HTMLからデータを取得する
 *
 * @param {string} parentTbodyId - 親テーブルID
 * @returns データ
 */
function getDataListFromTable(parentTbodyId) {
	let parentTable = document.querySelector("#" + parentTbodyId);
	let trList = parentTable.querySelectorAll("tr[data-row]");
	let retList  = [];
	trList.forEach(function(tr) {
		var data = {};
		let elems = tr.querySelectorAll("*[data-name]");
		elems.forEach(function(elem) {
			let name = elem.getAttribute("data-name");
			let value;
			if(["select", "input"].indexOf(elem.tagName.toLowerCase()) >= 0) {
				value = elem.value;
			}
			else {
				value = elem.textContent;
			}
			data[name] = value;
		});
		retList.push(data);
	});
	return retList;
}

/**
 * 入力コントロール有効無効切り替え
 *
 * @param {string} parentId - 親ID
 * @param {bool} disabled - 無効
 */
function setInputDisabled(parentId, disabled) {
	let parentElem = document.querySelector("#" + parentId);
	let elems = parentElem.querySelectorAll("*[data-name]");
	elems.forEach(function(elem) {
		if(["select", "input"].indexOf(elem.tagName.toLowerCase()) >= 0) {
			elem.disabled = disabled;
		}
	});
}

/**
 * DIV_Error表示
 *
 */
function showDivError() {
	document.getElementById('DIV_Error').style.display = '';
}

/**
 * DIV_Error非表示
 *
 */
function hideDivError() {
	document.getElementById('DIV_Error').style.display = 'none';
}

/**
 * Server_Error表示
 *
 * @param {string} message - メッセージ
 */
function showServerError(message) {
	let div = document.getElementById('Server_Error');
	div.style.display = '';

	let i = div.querySelector("i");
	div.textContent = null;
	if (i) {
		div.appendChild(i);
	}
	div.appendChild(document.createTextNode(message));
}

/**
 * Server_Error非表示
 *
 */
function hideServerError() {
    let div = document.getElementById('Server_Error');
    div.style.display = 'none';

	let i = div.querySelector("i");
	div.textContent = null;
	if (i) {
		div.appendChild(i);
	}
}

/**
 *ページ切り替え
 *
 * @param {string} page - ページ
 */
function changePage(page) {
    window.location.href = page;
}

/**
 * 権限取得
 *
 * @returns 権限
 */
function getPrivilege() {
	let priv = 0;
	let cookies = getCookies();
	if ("PRIVILEGE" in cookies) {
		priv = cookies["PRIVILEGE"];
	}
	return priv;
}

/**
 * 言語取得
 * @returns 言語
 */
function getLanguage() {
	let lang = "ja";
	let cookies = getCookies();
	if ("LANGUAGE" in cookies) {
		lang = cookies["LANGUAGE"];
	}
	return lang;
}

/**
 * ユーザー名取得
 *
 * @returns ユーザー名
 */
function getUser() {
	let cookies = getCookies();
	let user = ""
	if ("USER" in cookies) {
		user = cookies["USER"];
	}
	return user;
}

 
/**
 * 権限の表示を切り替える。
 *
 */
function showPriv() {
	let priv = getPrivilege();
	let elems = document.querySelectorAll(".priv_page");
	elems.forEach(function(elm) {
		if(elm.getAttribute("data-priv").split(",").indexOf(priv) >=0) {
			elm.style.display = "";
		}
		else {
			elm.style.display = "none";
		}
	});
}

/**
 * サブウィンドウを開く
 * @param {string} url - URL
 * @param {string} windowName - Window名
 * @param {int} subw - サブウインドウの横幅
 * @param {int} subh - サブウインドウの高さ
 */
function openSubWindow(url, windowName, subw, subh) {
	// 表示座標の計算
	let subx = ( screen.availWidth  - subw ) / 2;   // X座標
	let suby = ( screen.availHeight - subh ) / 2;   // Y座標
	// サブウインドウのオプション文字列を作る
	let SubWinOpt = "width=" + subw + ",height=" + subh + ",top=" + suby + ",left=" + subx;
	return window.open(url, windowName, SubWinOpt); 
 }
 /**
  * 画面全体のシェード
  * @param {bool} on - on(true)/off(off)
  */
function shade(on) {
	let s = document.getElementById("shadelayer");
	if (s) {
		if (on) {
			s.style.display = "block";
		}
		else {
			s.style.display = "none";
		}
		return s;
	}
}

/**
 * 子ウィンドウを監視し、子ウィンドウが開いている間は親ウィンドウを操作できないようにする。
 * @param {window} subwin -  子ウィンドウ
 */
function observeSubWin(subwin) {
	let s = shade(true);
	s.addEventListener("click", (e) => {
		if (subwin && subwin.closed == false) {
			shade(false);
			subwin.close();
			subwin = null;
		}
	});

	// １秒間隔で子画面の状態を監視
	let interval = setInterval(function()
	{
		// 子画面が閉じていたら
		if(!subwin || subwin.closed)
		{
			// 親画面のシェードを外す処理
			shade(false);

			// Intervalを破棄
			clearInterval(interval);
		// 画面が起動していたら
		}
		else
		{
			// 子画面にフォーカスを当てる
			if(!subwin.document.hasFocus())
			{
				subwin.focus();
			}
		}
	},500);

}
/**
 * メッセージのフォーマット
 * @param {string} format - 書式 ex. "message={0}"
 * @returns フォーマットされた文字列
 */
function formatMessage(format, args) {
	 if(arguments.length == 0) {
		 return;
	 }
	 else if(arguments.length == 1) {
		 return msg;
	 }
	 var args = Array.prototype.slice.call(arguments, 1);
	 return format.replace(/{(\d+)}/g, function(match, number) { 
		return typeof args[number] != 'undefined'? args[number] : match;
	});
 }

/**
 * 端末情報取得
 *
 * @returns Promise
 */
function getTerminalInfo() {
	return doPost("/get/terminalInfo", {}, handleLoginError)
	.then(function(resp) {
		return new Promise(function(resolve, reject) {
			if (resp.status == "ok") {
				resolve(resp.result);
			}
			else {
				reject();
			}
		});
	});
}

/**
 * ネットワーク情報取得
 *
 * @returns Promise
 */
function getNetworkInfo() {
	return doPost("/get/networkInfo", {}, handleLoginError)
	.then(function(resp) {
		return new Promise(function(resolve, reject) {
			if (resp.status == "ok") {
				resolve(resp.result);
			}
			else {
				reject();
			}
		});
	});
}
/**
 * ネットワーク情報取得
 *
 * @returns Promise
 */
function getNetworkSetting() {
	return doPost("/get/networkSetting", {}, handleLoginError)
	.then(function(resp) {
		return new Promise(function(resolve, reject) {
			if (resp.status == "ok") {
				resolve(resp.result);
			}
			else {
				reject();
			}
		});
	});
}

/**
 * その他設定情報取得
 *
 * @returns Promise
 */
function getOtherInfo() {
	return doPost("/get/otherInfo", {}, handleLoginError)
	.then(function(resp) {
		return new Promise(function(resolve, reject) {
			if (resp.status == "ok") {
				resolve(resp.result);
			}
			else {
				reject();
			}
		});
	});
}

/**
 * グループ取得
 *
 * @returns Promise
 */
function getGroupData() {
	return doPost("/get/groups", {}, handleLoginError)
	.then(function(resp) {
		return new Promise(function(resolve, reject) {
			if (resp.status == "ok") {
				resolve(resp.result);
			}
			else {
				reject();
			}
		});
	});
}

/**
 * 呼出機取得
 *
 * @returns Promise
 */
function getCallerData(sound) {
	return doPost("/get/callers", {"sound": sound}, handleLoginError)
	.then(function(resp) {
		return new Promise(function(resolve, reject) {
			if (resp.status == "ok") {
				resolve(resp.result);
			}
			else {
				reject();
			}
		});
	});
}

/**
 * 消去機取得
 *
 * @returns Promise
 */
 function getEraserData() {
	return doPost("/get/erasers", {}, handleLoginError)
	.then(function(resp) {
		return new Promise(function(resolve, reject) {
			if (resp.status == "ok") {
				resolve(resp.result);
			}
			else {
				reject();
			}
		});
	});
}

/**
 * イベント発火
 * @param {HTMLElement} elem - HTMLElement
 * @param {string} eventType - イベントタイプ "UIEvents", "MouseEvents", "MutationEvents", "HTMLEvents" 
 * @param {string} eventName - イベント名
 */
function fireEvent(elem, eventType,  eventName) {
	var event = document.createEvent( eventType ); // イベントオブジェクトを作成
	event.initEvent(eventName, true, true); // イベントの内容を設定
	return elem.dispatchEvent(event); // イベントを発火させる
}

/**
 * 文字列のバイト数（半角1byte,全角2byteとする）を取得
 * @param {string} str 文字列
 * @returns バイト数
 */
function lenb(str) {
	let len = 0;
  
	for (let i = 0; i < str.length; i++) {
	  (str[i].match(/[ -~｡-ﾟ]/)) ? len += 1 : len += 2;
	}
  
	return len;
}
/**
 * 文字列のバイト数（半角1byte,全角2byteとする）分切り出す
 * @param {string} str 文字列
 * @param {int} l 切り出すバイト数
 * @returns 切り出した文字列
 */
 function leftb(str, l) {
	let len = 0;
	let ret = "";
	for (let i = 0; i < str.length; i++) {
	  (str[i].match(/[ -~｡-ﾟ]/)) ? len += 1 : len += 2;
	  if(len <= l) {
		ret = ret + str[i];
	  }
	  else {
		  break;
	  }
	}
  
	return ret;
}

function handleLoginError() {
	changePage("login.html")
}