var MESSAGE_UPDCONFIRM = getResources("common_input:UPDCONFIRM");
var MESSAGE_JUMPCONFIRM = getResources("common_input:JUMPCONFIRM");

var CHANGE_FLAG = false;
/**
 * 登録ボタン押下イベント処理
 */
function funcUpdate() {
	if(funcInputCheck() == true) {
		setTimeout('funcConfirm()', 100);
	}
}

/**
 * 登録確認ダイアログ表示
 */
function funcConfirm() {
	if(confirm(MESSAGE_UPDCONFIRM)) {
        // document.forms[0].submit();
        doUpdate();
	}
}

/**
 * IPアドレスチェック
 * @param {string} value - 値 
 * @returns 結果
 */
function funcCheckIpAddress(value) {
	if (value != "") {
		var exp=/^(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])$/;
		if (value.match(exp) == null) {
			return false;
		}
	}
	return true;
}

/**
 * チェックエラーメッセージ累積
 * @param {string} msgOld - 旧メッセージ
 * @param {string} msgAdd - 追加メッセージ
 */
function funcAddMsg(msgOld, msgAdd) {
	if(msgOld == '') {
		return msgAdd;
	} else {
		return msgOld + '<br>' + msgAdd;
	}
}

/**
 * 画面編集時
 */
function funcChange() {
	CHANGE_FLAG = true;
}

/**
 * タブ切り替える時の処理
 * @param {string} view - 切り替えページ 
 */
function funcTabChange(view) {
	if(CHANGE_FLAG) {
		if(!confirm(MESSAGE_JUMPCONFIRM)) {
			return;
		}
	}
    window.location.href = view;
}

/**
 * 文字列チェック
 * 
 * @param {string} input - チェック対象文字列
 * @param {string} charType - チェック種別
 *                        　・"zenkaku"               : 全角文字（ひらがな・カタカナ・漢字 etc.）
 *                        　・"hiragana"              : 全角ひらがな
 *                        　・"katakana"              : 全角カタカナ
 *                        　・"alphanumeric"          : 半角英数字（大文字・小文字）
 *                        　・"numeric"               : 半角数字
 *                        　・"alphabetic"            : 半角英字（大文字・小文字）
 *                        　・"upper-alphabetic"      : 半角英字（大文字のみ）
 *                        　・"lower-alphabetic"      : 半角英字（小文字のみ）
 *                        　・"hankaku"               : 半角英数字記号
 * @return Boolean チェック結果OKかどうか 
 *               true  : チェックOK（引数に指定した種別の文字列のみで構成されている)
 *               false : チェックNG（引数に指定した種別以外の文字列が含まれている）
 */
function checkCharType(input, charType) {
    switch (charType) {
        // 全角文字（ひらがな・カタカナ・漢字 etc.）
        case "zenkaku":
            return (input.match(/^[^\x01-\x7E\xA1-\xDF]+$/)) ? true : false;
        // 全角ひらがな
        case "hiragana":
            return (input.match(/^[\u3041-\u3096]+$/)) ? true : false;
        // 全角カタカナ
        case "katakana":
            return (input.match(/^[\u30a1-\u30f6]+$/)) ? true : false;
        // 半角英数字（大文字・小文字）
        case "alphanumeric":
            return (input.match(/^[0-9a-zA-Z]+$/)) ? true : false;
        // 半角数字
        case "numeric":
            return (input.match(/^[0-9]+$/)) ? true : false;
        // 半角英字（大文字・小文字）
        case "alphabetic":
            return (input.match(/^[a-zA-Z]+$/)) ? true : false;
        // 半角英字（大文字のみ）
        case "upper-alphabetic":
            return (input.match(/^[A-Z]+$/)) ? true : false;
        // 半角英字（小文字のみ）
        case "lower-alphabetic":
            return (input.match(/^[a-z]+$/)) ? true : false;
        // 半角英数字記号
        case "hankaku":
            return (input.match(/[^\x01-\x7E]/)) ? false : true;
		// 0000～FFFF
		case "systemcode":
            return (input.match(/^[0-9a-fA-F]{4}$/)) ? true : false;
    }
    return false;
}

/**
 * エラー表示
 * @param {string} trId - trのid 
 * @param {string} pId - pのid
 * @param {string} message - メッセージ
 */
function setErrorOnTrById(trId, pId, message) {
    setErrorOnTr(document.getElementById(trId), document.getElementById(pId), message);
}
/**
 * エラークリア
 * @param {string} trId - trのid 
 * @param {string} pId - pのid
 */
function clearErrorOnTrById(trId, pId) {
    clearErrorOnTr(document.getElementById(trId), document.getElementById(pId));
}

/**
 * エラー表示
 * @param {HTMLElement} tr - tr element 
 * @param {HTMLElement} p - p element
 * @param {string} message -メッセージ
 */
function setErrorOnTr(tr, p, message) {
    if (tr) {
        tr.classList.add('gnp-error-point');
    }
    if (p) {
        p.style.display="";
        p.innerHTML = message;
    }
}
/**
 * エラークリア
 * @param {HTMLElement} tr - tr element 
 * @param {HTMLElement} p - p element
 */
function clearErrorOnTr(tr, p) {
    if (tr) {
        tr.classList.remove('gnp-error-point');
    }
    if (p) {
        p.style.display="none";
        p.innerHTML = '';
    }
}

/**
 * 入力コントロールにイベントリスナーを追加する
 * @param {string} parentId - 親ID 
 * @param {function} func - 追加するイベントリスナー
 */
function addOnChangeListnerToInput(parentId, func) {
    let elems = document.querySelectorAll("#" + parentId + " *[data-name]");
    elems.forEach((elm) => {
        if(["select", "input"].indexOf(elm.tagName.toLowerCase()) >= 0) {
            elm.addEventListener("change", func);
        }

    });

}