/**
 * POST
 * @param {string} url - URL 
 * @param {object} data - 送信データ
 */
function doPost(url, data, loginErrorFunc) {
    return new Promise(function(resolve, reject) {
        let xhr = new XMLHttpRequest();
        xhr.open("POST", url);
        xhr.setRequestHeader("Content-Type", "application/json");
        xhr.onreadystatechange = function() {
            if (xhr.readyState == 4 && xhr.status == 200) {
				try {
					let response = JSON.parse(xhr.responseText);
					if (response.status == "ok") {
						resolve(response);
					}
					else if("error" in response) {
						if(loginErrorFunc && response.error == "login") loginErrorFunc();
						reject(response.error);
					}
					else {
						reject()						
					}
				}
				catch (e) {
					reject(e);
				}
			}
        };
        xhr.onerror = function(e) {
            reject(e);
        };
        let json = JSON.stringify(data);
        xhr.send(json);
    });
}

/**
 * POST(ファイル送信用)
 * @param {string} url - URL 
 * @param {object} formData - Form Data 
 * @param {function} progress - onprogress callback 
 */
function doPostFile(url, formData, progress) {
    return new Promise(function(resolve, reject) {
        let xhr = new XMLHttpRequest();
        xhr.open("POST", url);
        xhr.onreadystatechange = function() {
			try {
				if (xhr.readyState == 4 && xhr.status == 200) {
					let response = JSON.parse(xhr.responseText);
					if (response.status == "ok") {
						resolve(response);
					}
					else if("error" in response) {
						reject(response.error);
					}
					else {
						reject()						
					}
				}
			}
			catch (e) {
				reject(e);
			}
		};
        xhr.onerror = function(e) {
            reject(e);
        };
        xhr.upload.onprogress = progress
        xhr.send(formData);
    });
}

/**
 * Cookie取得
 * @returns Cookie格納データ
 */
function getCookies() {
	let cookies = document.cookie;
	let cookiesArr = cookies.split(";");
	let cookieDict = {};
	for (let i = 0; i < cookiesArr.length; i++) {
		let cArr = cookiesArr[i].split("=");
		if(cArr.length == 2) {
			cookieDict[cArr[0].trim()] = cArr[1].trim(); 
		}
	}
	return cookieDict;
}
