
window.onload = funcOnload;

let delEraserList = [];
let orgEraserList = []
/**
 * ボディオンロードイベント処理
 *
 */
function funcOnload() {

	getTerminalInfo()
	.then((termInfo) => {
		setDictionaryData("headerTerminalInfo", termInfo);
	});

	getEraserData()
	.then((eraserData) => {
		let erasers = eraserData["erasers"];
		orgEraserList = erasers;
		createTableRow("tableBody", erasers);
	
		setDictionaryDataToTable("tableBody", erasers);

		addOnChangeListnerToInput("tableBody", funcChange);

		// 新規データの監視
		startWatchNewErasers(5000);

		document.getElementById('delEraserBtn').onclick = deleteErasers;
		document.getElementById('updateBtn').onclick = funcUpdate;
	});


}

/**
 * 新規消去機データの監視
 */
/**
 * 
 * @param {int} interval - 監視間隔（ミリ秒）
 */
function startWatchNewErasers(interval) {
	setInterval(()=> {
		getEraserData()
		.then((eraserData) => {
			// 新規データを取得
			let erasers = eraserData["erasers"];
			// 受信機で追加された消去機
			let newErasers = erasers.filter((e) => {
				// 既存のデータに一致するか
				let l =  orgEraserList.filter( oe => oe.address == e.address);
				if (l.length == 0) {
					// 不一致
					return true;
				}
				else {
					// 一致
					return false;
				}
			});
			// 受信機で削除された消去機
			let deletedErasers = orgEraserList.filter((oe) => {
				let l = erasers.filter( e => e.address == oe.address);
				if (l.length == 0) {
					// 不一致
					return true;
				}
				else {
					// 一致
					return false;
				}
			});

			// 受信機で削除された消去機を表示から削除する。
			if (deletedErasers.length > 0) {
				deleteErasersByReceiver(deletedErasers);
				orgEraserList = orgEraserList.filter((oe) => {
					let l = deletedErasers.filter(de => de.address == oe.address);
					if (l.length == 0) {
						return true;
					}
					else {
						return false;
					}
				});
			}
			// 受信機で追加された消去機を表示する。
			if (newErasers.length > 0) {
				insertErasers(newErasers);
				orgEraserList = orgEraserList.concat(newErasers)
			}

		});
	}, interval);
}

/**
 * 消去機を追加表示する。
 * @param {Array} newErasers - 新規追加された消去機 
 */
function insertErasers(newErasers) {
	let table = createDummyTable("dummyTable");
	document.body.appendChild(table);
	let row = orgEraserList.length;

	createTableRow("dummyTable", newErasers);
	
	setDictionaryDataToTable("dummyTable", newErasers);

	addOnChangeListnerToInput("dummyTable", funcChange);

	let tbody = document.querySelector("#tableBody");
	document.querySelectorAll("#dummyTable tr").forEach((tr) => {
		tr.setAttribute("data-row", row)
		tbody.appendChild(tr);
		row++;
	});

	updateDataRowNo();

	document.body.removeChild(table);
}

/**
 * 受信機で削除された消去機をテーブルから削除する。
 * @param {Array} deletedCallers - 受信機で削除された消去機 
 */
function deleteErasersByReceiver(deletedErasers) {
	deletedErasers.forEach((de) => {
		trList = findTrByAddress(de.address);
		trList.forEach((tr) => {
			tr.remove();
		});
	});

	updateDataRowNo();
}

/**
 * trのdata-row属性の値を振り直す
 */
function updateDataRowNo() {
	let tbody = document.querySelector("#tableBody");
	let cnt = 1;
	tbody.querySelectorAll("tr[data-row]").forEach((tr) => {
		tr.setAttribute("data-row", cnt);
		cnt++;
	});
}

/**
 * ダミーのテーブルを作成する
 * @param {string} tbodyId - tbodyのId
 */
function createDummyTable(tbodyId) {
	let table = document.createElement("table");
	table.style.display = "none";
	let tbody = document.createElement("tbody");
	tbody.setAttribute("id", tbodyId);
	table.appendChild(tbody);
	return table;
}

/**
 * 消去機表示テーブルの行を生成
 * 
 * @param {string} tbodyId - テーブルのID 
 * @param {Array} erasers - 消去機 
 */
function createTableRow(tbodyId, erasers) {
	let tbody = document.querySelector("#" + tbodyId);

	let row = 1;
	erasers.forEach((eraser) => {
		tr = createTr(row, eraser);
		tbody.appendChild(tr);
		row++;
	})
}

/**
 * TR要素生成
 * 
 * @param {int} row - 行
 * @param {object} eraser - 消去機 
 */
function createTr(row, eraser) {
	let tr = document.createElement("tr");
	tr.setAttribute("data-row", row);
	tr.classList.add("row-space");

	let tdBtn = createTd();
	tdBtn.classList.add("h-row");
	tdBtn.setAttribute("data-label", getResources("eraser:data-label:delete"));
	let delBtn = document.createElement('input');
	delBtn.setAttribute("type", "checkbox");
	delBtn.onclick = (e) => {
		// 削除チェックボックスクリック
		// e.preventDefault();
		let tr = e.target.parentNode.parentNode;
		let address = tr.querySelector("*[data-name='address']");
		if (tr.classList.contains('delete-row')) {
			// 削除済みは戻す
			findTrByAddress(address.textContent).forEach((tr) => {
				tr.classList.remove('delete-row');
				tr.querySelectorAll("input,select").forEach((elem) => {
					elem.disabled = false;
				});
	
			});

			delEraserList = delEraserList.filter( a => a != address.textContent);
		}
		else {
			// 行を灰色にする
			findTrByAddress(address.textContent).forEach((tr) => {
				tr.classList.add('delete-row');
				tr.querySelectorAll("input,select").forEach((elem) => {
					if (elem != e.target) {
						elem.disabled = true;
					}
				});
			});
			// 削除リストに追加する。
			delEraserList.push(address.textContent);
		}
		document.getElementById("delEraserBtn").disabled = !hasDeleteErasers();
	}
	tdBtn.appendChild(delBtn);

	let tdAddress = createTd();
	tdAddress.setAttribute("data-label", getResources("eraser:data-label:address"));
	["address", "modelNo"].forEach((dataName) => {
		let p = document.createElement("p");
		p.setAttribute("data-name", dataName);
		if (dataName == "modelNo") {
			p.classList.add("bracket");
		}
		tdAddress.appendChild(p);
	});

	let tdEraserName = document.createElement("td");
	tdEraserName.setAttribute("data-label", getResources("eraser:data-label:eraserName"));
	let input = createInput({"maxlength":20, "style": "width:160px;"}, "name", 20)
	tdEraserName.appendChild(input);

	tr.appendChild(tdBtn);
	tr.appendChild(tdAddress);
	tr.appendChild(tdEraserName);

	return tr;
}


/**
 * TD Element生成
 * @param {string} dataName - データ名
 * @returns TD Element
 */
function createTd(dataName) {
	let td = document.createElement("td");
	td.classList.add('inputs');
	if(dataName) {
		td.setAttribute("data-name", dataName);
	}

	return td;
}

/**
 * Input Element生成
 * @param {object} attrs - 属性
 * @param {string}} dataName - データ名
 * @return Input Element
 */
function createInput(attrs, dataName, lenbyte) {
	let input = document.createElement('input');
	input.setAttribute("type", "text");
	input.classList.add('gnp-input-text');
	// input.classList.add('input-s');
	// input.style.width = "100%";
	if(dataName) {
		input.setAttribute("data-name", dataName);
	}
	for (let  key in attrs) {
		input.setAttribute(key, attrs[key]);
	}
	if(lenbyte) {
		input.addEventListener('change', (ev) => {
			ev.preventDefault();
			let lb = lenb(input.value);
			if(lb > lenbyte) {
				input.value = leftb(input.value, lenbyte);
			}
		});
	}
	return input;
}


/**
 * アドレスが一致するTRを探す
 * @param {string} address - アドレス
 * @returns TRのList
 */
function findTrByAddress(address) {
	let trList = [];
	document.querySelectorAll('p[data-name="address"]').forEach((elem) => {
		if(elem.textContent == address) {
			trList.push(elem.parentNode.parentNode);
		}
	});
	return trList;
}



/**
 * 入力項目チェック
 * @returns 結果
 */
function funcInputCheck() {
	let bCheckResult = true;

	// 何かやることがあれば・・・

	if(bCheckResult == false) {
		showDivError();
		return false;
	} else {
		hideDivError();
		return true;
	}
}

/**
 * 入力コントロールの無効・有効
 * @param {boolean} disabled - 無効
 */
function disableControls(disabled) {
	setInputDisabled("tableBody", disabled);
	document.getElementById("delEraserBtn").disabled = disabled;
	document.getElementById("updateBtn").disabled = disabled;
}

/**
 * 更新実行
 */
function doUpdate() {
	let updateData = getDataListFromTable("tableBody");

	let param = {
		"delete": delEraserList,
		"update": updateData
	};
	console.log(param);

	disableControls(true);

	doPost("/update/erasers", param, handleLoginError)
	.then( (resp) => {
		if (resp.status == "ok") {
			location.reload();
		}
		else {
			showServerError(getResources("eraser:UPDATE_FAILED"));
			disableControls(false);
		}
	})
	.catch( () => {
		showServerError(getResources("eraser:UPDATE_FAILED"));
		disableControls(false);
	});
}

/**
 * 削除チェックをいれた消去機が存在するか
 * @return true:存在する、false:存在しない。
 */
function hasDeleteErasers() {
	let parentTable = document.getElementById("tableBody");
	let trList = parentTable.querySelectorAll("tr.delete-row[data-row]");
	if (trList.length == 0) {
		return false;
	}
	return true;
}

/**
 * 消去機削除
 */
function deleteErasers() {
	if (!hasDeleteErasers()) {
		return;
	}
	if (!confirm(getResources("eraser:DELETE_CONFIRM"))) {
		return;
	}

	let param = {
		"delete": delEraserList,
	};

	console.log(param);
	disableControls(true);
	doPost("/delete/erasers", param, handleLoginError)
	.then((resp) => {
		if (resp.status == "ok") {
			let parentTable = document.querySelector("#tableBody");
			let trList = parentTable.querySelectorAll("tr.delete-row[data-row]");
			trList.forEach(tr => {
				tr.remove();
			});
			delEraserList.forEach(address =>{
				orgEraserList = orgEraserList.filter(oe => oe.address != address)
			});
			delEraserList.length = 0;
			disableControls(false);
			document.getElementById("delEraserBtn").disabled = !hasDeleteErasers();
		}
		else {
			showServerError(getResources("eraser:DELETE_FAILED"));
			disableControls(false);
		}
	})
	.catch(() => {
		showServerError(getResources("eraser:DELETE_FAILED"));
		disableControls(false);
	});
}
