

window.onload = funcOnload;
let delGroupList = [];
/**
 * ボディオンロードイベント処理
 *
 */
function funcOnload() {

	getTerminalInfo()
	.then((termInfo) => {
		setDictionaryData("headerTerminalInfo", termInfo);
	});

	getGroupData()
	.then((groups) => {
		createTableRow(groups.length);
		setDictionaryDataToTable("tableBody", groups);

		addOnChangeListnerToInput("tableBody", funcChange);

		document.getElementById('addGroupBtn').onclick = addGroup;
		document.getElementById('delGroupBtn').onclick = deleteGroups;
		document.getElementById('updateBtn').onclick = funcUpdate;
	});


}

/**
 * グループテーブルの行を生成
 * @param {int} rowlen - 行数 
 */
function createTableRow(rowlen) {
	let tbody = document.querySelector("#tableBody");

	for (let i = 1; i <= rowlen; i++) {
		let tr = createGroupTr(i, false);
	
		tbody.appendChild(tr);
	}	
}

/**
 * TR Element生成
 * @param {int} row - 行
 * @returns TR Element
 */
function createGroupTr(row) {
	let tr = document.createElement("tr");
	tr.setAttribute("data-row", row);
	tr.classList.add("row-space");
	tr.setAttribute("data-update", "");
	let tdBtn = document.createElement("td");
	let tdId = document.createElement("td");
	let tdName = document.createElement("td");

	tdBtn.classList.add('inputs');
	tdBtn.setAttribute("data-label", getResources("group:data-label:delete"));
	tdBtn.classList.add("h-row");
	let delBtn = document.createElement('input');
	delBtn.setAttribute("type", "checkbox");
	delBtn.onclick = (e) => {
		// 削除チェックボックスクリック
		// e.preventDefault();
		let checked = e.target.checked;
		let tr = e.target.parentNode.parentNode;
		let groupId = tr.querySelector("*[data-name='groupId']");
		if (tr.classList.contains('delete-row')) {
			// 削除済みは戻す
			tr.classList.remove('delete-row');
			tr.querySelectorAll("input").forEach((elem) => {
				elem.disabled = false;
			});

			delGroupList = delGroupList.filter( id => id != groupId.textContent);
		}
		else {
			// 行を灰色にする
			tr.classList.add('delete-row');
			tr.querySelectorAll("input").forEach((elem) => {
				if (elem != e.target) {
					elem.disabled = true;
				}
			});
			// 削除リストに追加する。
			delGroupList.push(groupId.textContent);
		}
		document.getElementById("delGroupBtn").disabled = !hasDeleteGroups();
	};
	tdBtn.appendChild(delBtn);

	tdId.setAttribute("data-label", getResources("group:data-label:groupId"));
	tdId.classList.add('inputs');
	// グループID入力不可とする。
	tdId.setAttribute("data-name", "groupId");


	tdName.setAttribute("data-label", getResources("group:data-label:groupName"));
	tdName.classList.add('inputs');
	let inputName = document.createElement('input');
	inputName.setAttribute("type", "text");
	inputName.setAttribute("data-name", "name");
	// inputName.setAttribute("maxlength", "17");
	inputName.setAttribute("maxlength", "34");
	inputName.classList.add('gnp-input-text');
	lenbyte = 34;
	inputName.onchange = (ev) => {
		ev.preventDefault();
		let lb = lenb(inputName.value);
		if(lb > lenbyte) {
			inputName.value = leftb(inputName.value, lenbyte);
		}
	}
	let pName =document.createElement('p');
	pName.id = "P_name_" + row;  
	pName.classList.add("gnp-error-notice");
	pName.style.display = "none";
	tdName.appendChild(inputName);
	tdName.appendChild(pName);

	tr.appendChild(tdBtn);
	tr.appendChild(tdId);
	tr.appendChild(tdName);
	return tr;
}


/**
 * 入力項目チェック
 * @returns 結果
 */
function funcInputCheck() {
	let bCheckResult = true;

	let parentTable = document.querySelector("#tableBody");
	let trList = parentTable.querySelectorAll("tr[data-row]");
	trList.forEach((tr) => {
		let row = tr.getAttribute("data-row");
		let elems = tr.querySelectorAll("input[data-name]");
		elems.forEach((elem) => {
			let name = elem.getAttribute("data-name");
			let value = elem.value;
			if(name == "groupId") {
				clearErrorOnTrById("", "P_groupId_" + row);
				if (value.length == 0) {
					setErrorOnTrById("", "P_groupId_" + row, getResources("group:GROUPID_ERROR_REQUIRED"));
					bCheckResult = false;
				}
				else if (checkCharType(value, "numeric") == false) {
					setErrorOnTrById("", "P_groupId_" + row, getResources("group:GROUPID_ERROR_NUMERIC"));
					bCheckResult = false;
				}
			}
			else {
				// "name"
				clearErrorOnTrById("", "P_name_" + row);
				if (value.length == 0) {
					setErrorOnTrById("", "P_name_" + row, getResources("group:GROUPNAME_ERROR_REQUIRED"));
					bCheckResult = false;
				}
			}
		});
	});

	if(bCheckResult == false) {
		showDivError();
		return false;
	} else {
		hideDivError();
		return true;
	}
}

/**
 * 入力コントロールの無効・有効
 * @param {boolean} disabled - 無効
 */
function disableControls(disabled) {
	setInputDisabled("tableBody", disabled);
	document.getElementById("addGroupBtn").disabled = disabled;
	document.getElementById("delGroupBtn").disabled = disabled;
	document.getElementById("updateBtn").disabled = disabled;
}

/**
 * 更新
 */
function doUpdate() {
	let param = {
		"delete": delGroupList,
		"update": getUpdateData()
	};

	console.log(param);
	disableControls(true);
	doPost("/update/groups", param, handleLoginError)
	.then((resp) => {
		if (resp.status == "ok") {
			location.reload();
		}
		else {
			showServerError(getResources("group:UPDATE_FAILED"));
			disableControls(false);
		}
	})
	.catch(() => {
		showServerError(getResources("group:UPDATE_FAILED"));
		disableControls(false);
	});
}

/**
 * 削除チェックをいれたグループが存在するか
 * @return true:存在する、false:存在しない。
 */
function hasDeleteGroups() {
	let parentTable = document.getElementById("tableBody");
	let trList = parentTable.querySelectorAll("tr.delete-row[data-row]");
	if (trList.length == 0) {
		return false;
	}
	return true;
}
/**
 * グループ削除
 */
function deleteGroups() {
	if (!hasDeleteGroups()) {
		return;
	}		

	if (!confirm(getResources("group:DELETE_CONFIRM"))) {
		return;
	}

	let param = {
		"delete": delGroupList,
	};

	console.log(param);
	disableControls(true);
	doPost("/delete/groups", param, handleLoginError)
	.then((resp) => {
		if (resp.status == "ok") {
			// location.reload();
			let parentTable = document.querySelector("#tableBody");
			let trList = parentTable.querySelectorAll("tr.delete-row[data-row]");
			trList.forEach(tr => {
				tr.remove();
			});
			delGroupList.length = 0;
			disableControls(false);
			document.getElementById("delGroupBtn").disabled = !hasDeleteGroups();
		}
		else {
			showServerError(getResources("group:DELETE_FAILED"));
			disableControls(false);
		}
	})
	.catch(() => {
		showServerError(getResources("group:DELETE_FAILED"));
		disableControls(false);
	});
}

/**
 * 更新データ取得
 * @returns 更新データ
 */
function getUpdateData() {
	let parentTable = document.querySelector("#tableBody");
	let trList = parentTable.querySelectorAll("tr[data-row][data-update]");
	let retList  = [];
	trList.forEach((tr) => {
		var data = {};
		let elems = tr.querySelectorAll("*[data-name]");
		elems.forEach((elem) => {
			let name = elem.getAttribute("data-name");
			if(["input"].indexOf(elem.tagName.toLowerCase()) >= 0) {
				data[name] = elem.value;
			}
			else {
				data[name] = elem.textContent;
			}
		});
		retList.push(data);
	});
	return retList;
}


/**
 * グループ追加Window起動
 */
function addGroup() {
	let subwin = openSubWindow("add_group.html", "add_group", 600, 600);
	observeSubWin(subwin);
}

/**
 * グループ追加完了
 * @param {object} groupInfo 
 */
function addGroupFinished(groupInfo) {
	console.log(groupInfo);
	addGroupTr(groupInfo)
}

/**
 * グループ行を追加する
 * @param {object} groupInfo 
 */
function addGroupTr(groupInfo) {
	let tbody = document.querySelector("#tableBody");
	let trList = tbody.querySelectorAll("tr[data-row]");
	if (trList.length > 0) {
		n = parseInt(trList[trList.length - 1].getAttribute("data-row")) + 1;
	}
	else {
		n = 1;
	}
	let newTr = createGroupTr(n);
	newTr.querySelector('[data-name="groupId"]').textContent = groupInfo["groupId"];
	newTr.querySelector('[data-name="name"]').value = groupInfo["name"];
	tbody.appendChild(newTr);
}
