window.onload = funcOnload;
let ssidList = [];
let orgNetworkSetting; 
let selectedSsid;
/**
 * ボディオンロードイベント処理
 *
 */
function funcOnload() {

	// 端末情報取得
	getTerminalInfo()
	.then((termInfo) => {
		setDictionaryData("headerTerminalInfo", termInfo);
	});

	// ネットワーク設定取得
	getNetworkSetting()
	.then((networkSetting) => {
		setDictionaryData("wlanNetworkInfo", networkSetting.wlan);
		setRadioButton("wlan", networkSetting.wlan.enabled, networkSetting.wlan.autoIp, networkSetting.wlan.autoDNS);
		setRadioButton("eth",  networkSetting.eth.enabled, networkSetting.eth.autoIp, networkSetting.eth.autoDNS);

		setDictionaryData("ethNetworkInfo", networkSetting.eth);

		orgNetworkSetting = networkSetting;
		if (networkSetting.wlan.enabled == true) {
			setSsidProcessing(true);
			// SSIDリスト取得	
			return getSsidList()
		}
	})
	.then((res) => {
		ssidList = res;
		selectSsid(ssidList, orgNetworkSetting.wlan.ssid);
		updateSsidTable(ssidList);
		setSsidProcessing(false);
	})
	.catch(() => {
		setSsidProcessing(false);
	});


	document.getElementById('updateBtn').onclick = funcUpdate;
	document.getElementById('scanwifiBtn').onclick = scanWifi;

	addOnChangeListnerToInput("ethNetworkInfo", funcChange);
	addOnChangeListnerToInput("wlanNetworkInfo", funcChange);

}

/**
 * SSIDのListをscanする。
 */
function scanWifi() {
	clearErrorOnTrById("TR_wlan_ssid", "P_wlan_ssid");
	let tbody = document.querySelector("#ssidTbody");
	tbody.innerHTML = "";
	setSsidProcessing(true);
	getNetworkSetting()
	.then((networkSetting) => {
		orgNetworkSetting.wlan.ssid = networkSetting.wlan.ssid;
		orgNetworkSetting.wlan.ssidSettings = networkSetting.wlan.ssidSettings;
		return 	getSsidList();
	})
	.then((res) => {
		ssidList = res;
		selectSsid(ssidList, orgNetworkSetting.wlan.ssid);
		updateSsidTable(ssidList);
		setSsidProcessing(false);
	})
	.catch(() => {
		setSsidProcessing(false);
	});
}

/**
 * SSIDの選択状態を更新する。
 * @param {Array} ssidList - SSIDリスト 
 * @param {string} ssid - SSID
 */
function selectSsid(ssidList, ssid) {
	for(let s of ssidList) {
		if (s["essid"]== ssid) {
			s.selected = true;
		}
		else {
			s.selected = false;
		}
	}
}
/**
 * SSIDのリストを更新する。
 */
function updateSsidTable(ssidList) {
	let tbody = document.querySelector("#ssidTbody");
	tbody.innerHTML = "";
	ssidList.forEach( (ssidInfo) => {
		let tr = createSsidTr(ssidInfo, orgNetworkSetting.wlan);
		tbody.appendChild(tr);
	});
}
/**
 * 処理中イメージ表示
 * @param {bool} processing - 処理中 
 */
function setSsidProcessing(processing) {
	if(processing) {
		document.getElementById("ssid_processing").style.display = "block";
	}
	else {
		document.getElementById("ssid_processing").style.display = "none";
	}
}
/**
 * SSID一覧テーブルのTR生成
 * @param {object} ssidInfo - SSID情報
 * @param {object} wlanInfo - 無線LANネットワーク情報
 * @return tr Element
 */
function createSsidTr(ssidInfo, wlanInfo) {
	let tr = document.createElement("tr");
	tr.setAttribute("data-ssid", ssidInfo["essid"]);

	// 電波強度
	let tdLevel = createSignalLebelTd(ssidInfo);
	// SSID
	let tdName = createSsidNameTd(ssidInfo);
	// 暗号
	let tdEnc = createEncyptTd(ssidInfo);
	// 接続情報
	let tdInfo = createConnectInfoTd(ssidInfo, wlanInfo);
	// 選択中
	let tdSelect = createSelectedTd(ssidInfo);

	tr.appendChild(tdLevel);
	tr.appendChild(tdName);
	tr.appendChild(tdEnc);
	tr.appendChild(tdInfo);
	tr.appendChild(tdSelect);
	
	tr.onclick = (e) => {
		e.preventDefault();
		let ssid = e.currentTarget.getAttribute("data-ssid");
		if (hasSsidSetting(wlanInfo, ssid)) {
			selectedSsid = ssid;
			selectSsid(ssidList, ssid);
			updateSsidTable(ssidList);
		}
		else {
			setupSSid(ssid);
		}
	};

	return tr;
}

/**
 * 電波強度のTDを作成する
 * @param {object} ssidInfo - SSID情報
 * @return td Element
 */
function createSignalLebelTd(ssidInfo) {
	// 信号強度	
	// -30 dBm	素晴らしい	達成可能な最大信号強度。 クライアントは、これを実現するには、APから僅か数フィートである必要があります。現実的には、一般的ではなく望ましいものでもありません	N/A
	// -67 dBm	良好	非常に信頼性の高く、データパケットのタイムリーな伝送を必要とするアプリケーションのための最小信号強度	VoIP/VoWiFi, ストリーミングビデオ
	// -70 dBm	Okay	信頼できるパケット伝送に必要な最小信号強度	Email, web
	// -80 dBm	よくない	基本的なコネクティビティに必要な最小信号強度。パケット伝送は信頼できない可能性があります	N/A
	// -90 dBm	使用不可	ノイズレベルに近いかそれ以下の信号強度。殆ど機能しない	N/A	
	signalLevel = parseInt(ssidInfo["signal_level_dBm"]);
	let signalImg = "";
	if (signalLevel > -67) {
		signalImg = "img/network/icon_ssid_wifi_4.png";
	}
	else if (signalLevel > -70) {
		signalImg = "img/network/icon_ssid_wifi_3.png";
	}
	else if (signalLevel > -80) {
		signalImg = "img/network/icon_ssid_wifi_2.png";
	}
	else if (signalLevel > -90) {
		signalImg = "img/network/icon_ssid_wifi_1.png";
	}
	else  {
		signalImg = "img/network/icon_ssid_wifi_0.png";
	}

	let td = document.createElement("td");
	let img = document.createElement("img");
	img.setAttribute("src", signalImg);
	td.appendChild(img);

	return td;
}

/**
 * 鍵のTDを作成する
 * @param {object} ssidInfo - SSID情報
 * @return td Element
 */
function createEncyptTd(ssidInfo) {
	let td = document.createElement("td");
	if (ssidInfo.encryptions.length > 0) {
		let img = document.createElement("img");
		img.setAttribute("src", "img/network/icon_ssid_key.png");
		td.appendChild(img);
	}
	return td;
}

/**
 * SSIDのTDを作成する
 * @param {object} ssidInfo - SSID情報
 * @return td Element
 */
function createSsidNameTd(ssidInfo) {
	let td = document.createElement("td");
	td.textContent = ssidInfo["essid"];
	return td;
}

/**
 * 接続情報のTDを作成する
 * @param {object} ssidInfo - SSID情報
 * @param {object} wlanInfo - 無線LAN設定情報
 * @return td Element
 */
function createConnectInfoTd(ssidInfo, wlanInfo) {
	let td = document.createElement("td");
	if (hasSsidSetting(wlanInfo, ssidInfo["essid"])) {
		let btn = document.createElement("button");
		let img = document.createElement("img");
		img.setAttribute("src", "img/network/icon_ssid_info.png");
		btn.appendChild(img);
		btn.setAttribute("data-ssid", ssidInfo["essid"]);
		btn.onclick = (e) => {
			e.stopPropagation();
			let ssid = e.currentTarget.getAttribute("data-ssid");
			setupSSid(ssid);
		};
		td.appendChild(btn);
	}
	return td;
}

/**
 * 選択中SSIDのTDを作成する
 * @param {object} ssidInfo - SSID情報
 * @return td Element
 */
function createSelectedTd(ssidInfo) {
	let td = document.createElement("td");
	if (ssidInfo["selected"]) {
		td.innerHTML = "<b>✓</b>";
	}
	return td;
}

/**
 * SSID設定情報を持っているか
 * @param {object} wlanInfo - 無線LAN設定情報 
 * @param {string} ssid - SSID
 * @returns 結果
 */
function hasSsidSetting(wlanInfo, ssid) {
	return wlanInfo.ssidSettings && wlanInfo.ssidSettings[ssid];
}
/**
 * Radio Buttonの設定
 * @param {string} type - eth（有線LAN） or wlan（無線LAN）
 * @param {boolean} enabled - 有効
 * @param {boolean} autoIp - 自動IPアドレス
 * @param {boolean} autoDNS - 自動DNS
 */
function setRadioButton(type, enabled, autoIp, autoDNS) {
	let parentId = type + "NetworkInfo";

	let rdoEnabled1 = getDataElem(parentId, "enabled1");
	let rdoEnabled2 = getDataElem(parentId, "enabled2");

	rdoEnabled1.onclick = () => { 
		changeNetworkControls(type);
		if (type == "wlan") {
			scanWifi();
		}
	};
	rdoEnabled2.onclick = () => { 
		changeNetworkControls(type);
		if (type == "wlan") {
			let tbody = document.querySelector("#ssidTbody");
			tbody.innerHTML = "";
		}
	};

	if (enabled == true) {
		rdoEnabled1.checked = true;
		rdoEnabled2.checked = false;
		changeNetworkControls(type);
	}
	else {
		rdoEnabled1.checked = false;
		rdoEnabled2.checked = true;
		changeNetworkControls(type);
	}

	if (type == "eth") {
		let rdoAutoIp1 = getDataElem(parentId, "autoIp1");
		let rdoAutoIp2 = getDataElem(parentId, "autoIp2");
	
		rdoAutoIp1.onclick = () => { funcClickDHCP(false, true); };
		rdoAutoIp2.onclick = () => { funcClickDHCP(false, false); };
	
		if (autoIp == true) {
			rdoAutoIp1.checked = true;
			rdoAutoIp2.checked = false;
			funcClickDHCP(false, true);
		}
		else {
			rdoAutoIp1.checked = false;
			rdoAutoIp2.checked = true;
			funcClickDHCP(false, false);
		}
	
		let rdoAutoDNS1 = getDataElem(parentId, "autoDNS1");
		let rdoAutoDNS2 = getDataElem(parentId, "autoDNS2");
		rdoAutoDNS1.onclick = () => { funcClickDHCP(true, true); };
		rdoAutoDNS2.onclick = () => { funcClickDHCP(true, false); };
		if (autoDNS == true) {
			rdoAutoDNS1.checked = true;
			rdoAutoDNS2.checked = false;
			funcClickDHCP(true, true);
		}
		else {
			rdoAutoDNS1.checked = false;
			rdoAutoDNS2.checked = true;
			funcClickDHCP(true, false);
		}
	}
}

/**
 * DHCP変更時処理
 * @param {boolean} dns - true:DNS、false:IP 
 * @param {boolean} flag  - true:自動、false:固定
 */
function funcClickDHCP(dns, flag) {

	let parentId = "ethNetworkInfo";
	let ip = getDataElem(parentId, "ipAddress");
	let sub = getDataElem(parentId, "subnetMask");
	let gw = getDataElem(parentId, "defaultGw");
	let dns1 = getDataElem(parentId, "dns1");
	let dns2 = getDataElem(parentId, "dns2");

	var gray = '#808080';
	if(dns == true) {
		if(flag) {
			//背景色をグレーに
			dns1.style.backgroundColor = gray;
			dns2.style.backgroundColor = gray;
		}
		else {
			//背景色を白に
			dns1.style.backgroundColor = '';
			dns2.style.backgroundColor = '';
		}
	}
	else {
		if(flag) {
			//背景色をグレーに
			ip.style.backgroundColor = gray;
			sub.style.backgroundColor = gray;
			gw.style.backgroundColor = gray;
		}
		else {
			//背景色を白に
			ip.style.backgroundColor = '';
			sub.style.backgroundColor = '';
			gw.style.backgroundColor = '';
		}
	}

}

/**
 * 入力項目チェック
 * @returns 結果
 */
function funcInputCheck() {
	var bCheckResult = true;


	clearErrorOnTrById("TR_eth_ipAddress", "P_eth_ipAddress");
	clearErrorOnTrById("TR_eth_subnetMask", "P_eth_subnetMask");
	clearErrorOnTrById("TR_eth_defaultGw", "P_eth_defaultGw");
	clearErrorOnTrById("TR_eth_dns1", "P_eth_dns1");
	clearErrorOnTrById("TR_eth_dns2", "P_eth_dns2");
	let ethEnabled = getDataElem("ethNetworkInfo", "enabled1");
	if(ethEnabled.checked == true) {
		//有線LAN IPアドレス
		let ethAutoIp = getDataElem("ethNetworkInfo", "autoIp1");
		let ethIpAddress = getDataElem("ethNetworkInfo", "ipAddress");
		if(ethAutoIp.checked == false) {
			if(ethIpAddress.value == '') {
				setErrorOnTrById("TR_eth_ipAddress", "P_eth_ipAddress", getResources("network:IPADDR_ERROR_REQUIRED"));
				bCheckResult = false;
			}
			else if (funcCheckIpAddress(ethIpAddress.value) == false) {
				setErrorOnTrById("TR_eth_ipAddress", "P_eth_ipAddress", getResources("network:IPADDR_ERROR_FORMAT"));
				bCheckResult = false;
			}
		}

		//有線LAN サブネットマスク
		let ethSubnetMask = getDataElem("ethNetworkInfo", "subnetMask");
		if(ethAutoIp.checked == false) {
			if(ethSubnetMask.value == '') {
				setErrorOnTrById("TR_eth_subnetMask", "P_eth_subnetMask", getResources("network:SUBNET_ERROR_REQUIRED"));
				bCheckResult = false;
			}
			else if (funcCheckIpAddress(ethSubnetMask.value) == false) {
				setErrorOnTrById("TR_eth_subnetMask", "P_eth_subnetMask", getResources("network:SUBNET_ERROR_FORMAT"));
				bCheckResult = false;
			}
		}

		//有線LAN デフォルトGW
		let ethDefaultGw = getDataElem("ethNetworkInfo", "defaultGw");
		if(ethAutoIp.checked == false) {
			if (ethDefaultGw.value != "" && funcCheckIpAddress(ethDefaultGw.value) == false) {
				setErrorOnTrById("TR_eth_defaultGw", "P_eth_defaultGw", getResources("network:GW_ERROR_FORMAT"));
				bCheckResult = false;
			}
		}

		//有線LAN DNS
		let ethAutoDNS = getDataElem("ethNetworkInfo", "autoDNS1");
		let ethDNS1 = getDataElem("ethNetworkInfo", "dns1");
		if (ethAutoDNS.checked == false) {
			if (ethDNS1.value != "" && funcCheckIpAddress(ethDNS1.value) == false) {
				setErrorOnTrById("TR_eth_dns1", "P_eth_dns1", getResources("network:DNS1_ERROR_FORMAT"));
				bCheckResult = false;
			}
		}
		let ethDNS2 = getDataElem("ethNetworkInfo", "dns2");
		if (ethAutoDNS.checked == false) {
			if (ethDNS2.value != "" && funcCheckIpAddress(ethDNS2.value) == false) {
				setErrorOnTrById("TR_eth_dns2", "P_eth_dns2", getResources("network:DNS1_ERROR_FORMAT"));
				bCheckResult = false;
			}
		}
	}

	let wlanEnabled = getDataElem("wlanNetworkInfo", "enabled1");
	clearErrorOnTrById("TR_wlan_ssid", "P_wlan_ssid");
	if(wlanEnabled.checked == true) {
		let selectedSsidInfo = ssidList.filter(s => s.selected == true);
		if (selectedSsidInfo.length == 0) {
			setErrorOnTrById("TR_wlan_ssid", "P_wlan_ssid", getResources("network:SSID_ERROR_REQUIRED"));
			bCheckResult = false;
		}
		else {
			let ssid = selectedSsidInfo[0]["essid"];
			if (!orgNetworkSetting.wlan.ssidSettings[ssid]) {
				setErrorOnTrById("TR_wlan_ssid", "P_wlan_ssid", formatMessage(getResources("network:SSID_ERROR_NO_CONFIG"), ssid));
				bCheckResult = false;
			}
		}
	}


	if(bCheckResult == false) {
		showDivError();
		return false;
	} else {
		hideDivError();
		return true;
	}
}

/**
 * 入力コントロールの無効・有効
 * @param {boolean} disabled - 無効
 */
function disableControls(disabled) {
	setInputDisabled("ethNetworkInfo", disabled);
	let ethEnabledElem1 = getDataElem("ethNetworkInfo", "enabled1");
	let ethEnabledElem2 = getDataElem("ethNetworkInfo", "enabled2");
	ethEnabledElem1.disabled = true;
	ethEnabledElem2.disabled = true;

	setInputDisabled("wlanNetworkInfo", disabled);
	document.getElementById("updateBtn").disabled = disabled;
	document.getElementById("scanwifiBtn").disabled = disabled;
	if (disabled == false) {
		let wlanEnabledElem1 = getDataElem("wlanNetworkInfo", "enabled1");
		if (wlanEnabledElem1.checked == false) {
			document.getElementById("scanwifiBtn").disabled = true;
		}
	}
}

/**
 * 入力コントロールの有効無効をラジオボタンの選択状態により切り替える
 * @param {string} type - "eth":有線LAN、"wlan":無線LAN 
 */
function changeNetworkControls(type) {
	let parentId = type + "NetworkInfo";
	let enabledElem1 = getDataElem(parentId, "enabled1");
	let enabledElem2 = getDataElem(parentId, "enabled2");
	if (enabledElem1.checked) {
		setInputDisabled(parentId, false);
		if(type == "wlan") {
			document.getElementById("scanwifiBtn").disabled = false;
		}
	}
	else {
		setInputDisabled(parentId, true);
		enabledElem1.disabled = false
		enabledElem2.disabled = false
		if(type == "wlan") {
			document.getElementById("scanwifiBtn").disabled = true;
		}
	}
	if(type == "eth") {
		enabledElem1.disabled = true
		enabledElem2.disabled = true
	}
}

/**
 * 更新
 */
function doUpdate() {
	let ethNetworkInfo = getDictionaryData("ethNetworkInfo");
	let wlanNetworkInfo = getDictionaryData("wlanNetworkInfo");

	let ethEnabled = getDataElem("ethNetworkInfo", "enabled1"); 
	let ethAutoIp = getDataElem("ethNetworkInfo", "autoIp1");
	let ethAutoDNS = getDataElem("ethNetworkInfo", "autoDNS1");
	ethNetworkInfo["enabled"] = ethEnabled.checked;
	ethNetworkInfo["autoIp"] = ethAutoIp.checked;
	ethNetworkInfo["autoDNS"] = ethAutoDNS.checked;

	let wlanEnabled = getDataElem("wlanNetworkInfo", "enabled1"); 
	wlanNetworkInfo["enabled"] = wlanEnabled.checked;

	delete ethNetworkInfo["enabled1"];
	delete ethNetworkInfo["enabled2"];
	delete ethNetworkInfo["autoIp1"];
	delete ethNetworkInfo["autoIp2"];
	delete ethNetworkInfo["autoDNS1"];
	delete ethNetworkInfo["autoDNS2"];
	delete wlanNetworkInfo["enabled1"];
	delete wlanNetworkInfo["enabled2"];

	let selectedSsidInfo = ssidList.filter(s => s.selected == true);
	if (selectedSsidInfo.length > 0) {
		wlanNetworkInfo["ssid"] = selectedSsidInfo[0]["essid"];
	}

	let param = {
		"eth": ethNetworkInfo,
		"wlan": wlanNetworkInfo,
	};

	disableControls(true);

	doPost("/update/networkSetting", param, handleLoginError)
	.then((resp) => {
		if (resp.status == "ok") {
			location.reload();
		}
		else {
			showServerError(getResources("network:UPDATE_FAILED"));
			disableControls(false);
		}
	})
	.catch( () => {
		showServerError(getResources("network:UPDATE_FAILED"));
		disableControls(false);
	});
}


/**
 * SSID情報の取得
 */
function getSsidList() {
	return doPost("/scan/wifi", {}, handleLoginError)
	.then((resp) => {
		return new Promise((resolve, reject) => {
			if (resp.status == "ok") {
				resolve(resp.result);
			}
			else {
				reject()
			}
		});
	});
	
}

/**
 * SSID設定Window起動
 * @param {string} ssid - SSID
 */
function setupSSid(ssid) {
	selectedSsid = ssid;
	let subwin = openSubWindow("ssid_setting.html", "ssid_setting", 600, 600);
	observeSubWin(subwin);
}

/**
 * 子WindowからSSID情報を取得するための関数
 * @returns SSID情報
 */
function getSsidParam() {
	let ssidInfoArr = ssidList.filter(s => s["essid"] == selectedSsid);
	let ssidInfo;
	if (ssidInfoArr.length > 0) {
		ssidInfo = ssidInfoArr[0];
		return ssidInfo;
	}
	return ssidInfo;
}

/**
 * 子Windowでの保存処理が正常に終了したときに呼ばれる関数
 * @param {string} ssid - SSID 
 */
function saveSsidSettingFinished(ssid) {
	selectSsid(ssidList, ssid);
	getNetworkSetting()
	.then((networkSetting) => {
		orgNetworkSetting.wlan.ssid = networkSetting.wlan.ssid;
		orgNetworkSetting.wlan.ssidSettings = networkSetting.wlan.ssidSettings;
		updateSsidTable(ssidList);
	})
}

/**
 * 子Windowの削除処理が正常に終了したときに呼ばれる関数
 * @param {string} ssid - SSID 
 */
function deleteSsidSettingFinished(ssid) {
	// チェックを外す
	for(let s of ssidList) {
		if (s["essid"]== ssid) {
			s.selected = false;
		}
	}

	getNetworkSetting()
	.then((networkSetting) => {
		orgNetworkSetting.wlan.ssid = networkSetting.wlan.ssid;
		orgNetworkSetting.wlan.ssidSettings = networkSetting.wlan.ssidSettings;
		updateSsidTable(ssidList);
	})

}