window.onload = funcOnload;
/**
 * ボディオンロードイベント処理
 *
 */
function funcOnload() {
	showPriv();

	getTerminalInfo()
	.then((termInfo) => {
		setDictionaryData("headerTerminalInfo", termInfo);
	});

	getOtherInfo()
	.then((otherInfo) => {
		setDictionaryData("sound", otherInfo.sound);
		setDictionaryData("timeSetting", otherInfo.timeSetting);
		setDictionaryData("callerA", otherInfo.caller.A);
		setRadioButton(otherInfo.timeSetting.ntp);
		let now = new Date();
		let dateInput = document.getElementById("manualDate");
		dateInput.value = formatDate(now, "yyyy-MM-dd");
		let timeInput = document.getElementById("manualTime");
		timeInput.value = formatDate(now, "HH:mm") + ":00";

		// setDictionaryData("logServer", otherInfo.logServer);

	});

	document.getElementById('updateBtn').onclick = funcUpdate;
	document.getElementById('ntpTestBtn').onclick = ntpTimeset;
	document.getElementById('manualTimesetBtn').onclick = manualTimeset;
	document.getElementById('uploadFirmBtn').onclick = uploadFirmware;
	// document.getElementById('sendLogTestBtn').onclick = sendLogTest;
	document.getElementById('dowloadLogBtn').onclick = downloadLog;

	addOnChangeListnerToInput("sound", funcChange);
	addOnChangeListnerToInput("timeSetting", funcChange);
	// addOnChangeListnerToInput("logServer", funcChange);
	if (getPrivilege() >= 2) {
		document.getElementById('factorySettingBtn').onclick = restoreFactorySetting;	
	}

	if (getPrivilege() == 9) {
		getSpecial()
		.then((specialInfo) => {
			let ssh = specialInfo["ssh"]
			let sshSelect = document.getElementById("sshSelect");
			sshSelect.value = ssh ? "1": "0"

			let logLevel = specialInfo["logLevel"];
			let logLevelSelect = document.getElementById("logLevelSelect");
			logLevelSelect.value = logLevel;
			
		});
	
		document.getElementById('initBtn').onclick = initData;	
		document.getElementById('rebootBtn').onclick = reboot;	
		document.getElementById('sshBtn').onclick = setSsh;	
		document.getElementById('logLevelBtn').onclick = setLogLevel;	
	}

}

/**
 * Radio Buttonの設定
 * @param {boolean} ntp - true:NTP有効、false:手動 
 */
function setRadioButton(ntp) {
	let rdoNtp1 = getDataElem("timeSetting", "ntp1");
	let rdoNtp2 = getDataElem("timeSetting", "ntp2");


	if (ntp == true) {
		rdoNtp1.checked = true;
		rdoNtp2.checked = false;
	}
	else {
		rdoNtp1.checked = false;
		rdoNtp2.checked = true;
	}
}

/**
 * 日時のフォーマット
 * @param {Date} date - 日時
 * @param {string} format - フォーマット
 * @return フォーマット後データ 
 */
function formatDate (date, format) {
	format = format.replace(/yyyy/g, date.getFullYear());
	format = format.replace(/MM/g, ('0' + (date.getMonth() + 1)).slice(-2));
	format = format.replace(/dd/g, ('0' + date.getDate()).slice(-2));
	format = format.replace(/HH/g, ('0' + date.getHours()).slice(-2));
	format = format.replace(/mm/g, ('0' + date.getMinutes()).slice(-2));
	format = format.replace(/ss/g, ('0' + date.getSeconds()).slice(-2));
	format = format.replace(/SSS/g, ('00' + date.getMilliseconds()).slice(-3));
	return format;
  };

/**
 * 入力項目チェック
 * @returns 結果
 */
function funcInputCheck() {
	var bCheckResult = true;

	//NTPサーバ
	let ntp = getDataElem("timeSetting", "ntp1");
	let ntpServer = getDataElem("timeSetting", "ntpServer");
	clearErrorOnTrById("TR_timeSetting_ntpServer", "P_timeSetting_ntpServer");
	if (ntp.checked == true) {
		if (ntpServer.value == "") {
			setErrorOnTrById("TR_timeSetting_ntpServer", "P_timeSetting_ntpServer", getResources("other:NTPSERVER_ERROR_REQUIRED"));
			bCheckResult = false;
		}
		else if(checkCharType(ntpServer.value, 'hankaku') == false) {
			setErrorOnTrById("TR_timeSetting_ntpServer", "P_timeSetting_ntpServer", getResources("other:NTPSERVER_ERROR_FORMAT"));
			bCheckResult = false;
		}
	}
	
	// //ログサーバー
	// let logIpAddress = getDataElem("logServer", "ipAddress");
	// clearErrorOnTrById("TR_logServer_ipAddress", "P_logServer_ipAddress");
	// if (logIpAddress.value == "") {
	// 	setErrorOnTrById("TR_logServer_ipAddress", "P_logServer_ipAddress", getResources("other:LOGSERVER_ERROR_REQUIRED"));
	// 	bCheckResult = false;
	// }
	// else if(funcCheckIpAddress(logIpAddress.value) == false) {
	// 	setErrorOnTrById("TR_logServer_ipAddress", "P_logServer_ipAddress", getResources("other:LOGSERVER_ERROR_FORMAT"));
	// 	bCheckResult = false;
	// }


	if(bCheckResult == false) {
		showDivError();
		return false;
	} else {
		hideDivError();
		return true;
	}
}

/**
 * 入力コントロールの無効・有効
 * @param {boolean} disabled - 無効
 */
function disableControls(disabled) {
	setInputDisabled("sound", disabled);
	setInputDisabled("timeSetting", disabled);
	// setInputDisabled("logServer", disabled);
	document.getElementById("updateBtn").disabled = disabled;
}

/**
 * ファームウェアアップロード用コントロールの無効・有効
 * @param {boolean} disabled - 無効
 */
function disableFirmUploadControls(disabled) {
	document.getElementById("uploadFile").disabled = disabled;	
	document.getElementById("uploadFirmBtn").disabled = disabled;	
}

/**
 * 特殊設定用コントロールの無効・有効
 * @param {boolean} disabled - 無効
 */
function disableSpecialControls(disabled) {
	setInputDisabled("special", disabled);
	document.getElementById('sshBtn').disabled = disabled;	
	document.getElementById('logLevelBtn').disabled = disabled;	
	document.getElementById('initBtn').disabled = disabled;	
	document.getElementById('rebootBtn').disabled = disabled;	
}

/**
 * 更新
 */
function doUpdate() {
	let sound = getDictionaryData("sound");
	let timeSetting = getDictionaryData("timeSetting");
	let callerA = getDictionaryData("callerA");
	// let logServer = getDictionaryData("logServer");
	sound.volume = parseInt(sound.volume, 10);
	delete timeSetting["ntp1"];
	delete timeSetting["ntp2"];
	timeSetting["ntp"] = getDataElem("timeSetting", "ntp1").checked;

	callerA.longPressTime = parseInt(callerA.longPressTime, 10);

	// let param = {"sound": sound, "timeSetting": timeSetting, "logServer": logServer};
	let param = {"sound": sound, "timeSetting": timeSetting, "caller": {"A": callerA}};

	disableControls(true);

	doPost("/update/otherInfo", param, handleLoginError)
	.then((resp) => {
		if (resp.status == "ok") {
			location.reload();
		}
		else {
			showServerError(getResources("other:UPDATE_FAILED"));
			disableControls(false);
		}
	})
	.catch( () => {
		showServerError(getResources("other:UPDATE_FAILED"));
		disableControls(false);
	});
}

/**
 * NTP時刻合わせ
 */
function ntpTimeset() {
	let ntpServerInput = getDataElem("timeSetting", "ntpServer");
	let param = {
		"type": "ntp",
		"server": ntpServerInput.value
	}

	doPost("/set/time", param, handleLoginError)
	.then((resp) => {
		if (resp.status == "ok") {
			alert(formatMessage(getResources("other:NTP_SUCCESS"), resp.result.time));
		}
		else {
			alert(getResources("other:NTP_FAILED"));
		}
	})
	.catch( () => {
		alert(getResources("other:NTP_FAILED"));
	});

}

/**
 * 手動時刻合わせ
 */
function manualTimeset() {
	let dateInput = document.getElementById("manualDate");
	let timeInput = document.getElementById("manualTime");
	let param = {
		"type": "manual",
		"date": dateInput.value.split("-").join("/"),
		"time": timeInput.value
	}

	doPost("/set/time", param, handleLoginError)
	.then((resp) => {
		if (resp.status == "ok") {
			alert(formatMessage(getResources("other:SET_TIME_SUCCESS"), resp.result.time));
		}
		else {
			alert(getResources("other:SET_TIME_FAILED"));
		}
	})
	.catch( () => {
		alert(getResources("other:SET_TIME_FAILED"));
	});

}

/**
 * ファームウェアアップロード
 */
function uploadFirmware() {
	let result = window.confirm(getResources("other:FW_UPLOAD_CONFIRM"));
	if (result) {
		let formdata = new FormData(document.getElementById("upload_form"));

		clearErrorOnTrById("TR_upload_firmware", "P_upload_firmware");
		disableFirmUploadControls(true)
		doPostFile("/upload/firmware",formdata, ()=>{})
		.then((resp) => {
			if (resp.status == "ok") {
				// alert("ファームウェアをアップロードしました。")
			}
			else {
				setErrorOnTrById("TR_upload_firmware", "P_upload_firmware", getResources("other:FW_UPLOAD_FAILED"));
			}
			disableFirmUploadControls(false)
		})
		.catch(()=> {
			setErrorOnTrById("TR_upload_firmware", "P_upload_firmware", getResources("other:FW_UPLOAD_FAILED"));
			disableFirmUploadControls(false)
		});
	}
}


// /**
//  * ログ送信テスト
//  */
// function sendLogTest() {
// 	let ipAddressInput = getDataElem("logServer", "ipAddress");
// 	let param = {
// 		"ipAddress": ipAddressInput.value
// 	}
// 	doPost("/test/logsend", param)
// 	.then((resp) => {
// 		if (resp.status == "ok") {
// 			alert(getResources("other:SEND_TEST_SUCCESS"));
// 		}
// 		else {
// 			alert(getResources("other:SEND_TEST_FAILED"));
// 		}
// 	})
// 	.catch(() => {
// 		alert(getResources("other:SEND_TEST_FAILED"));
// 	})
// }
/**
 * ログダウンロード
 */
function downloadLog() {

	document.getElementById("dowloadLogBtn").disabled = true;
	setDownloading(true, 0);
	doPost("/download/log", {}, handleLoginError)
	.then((resp) => {
		if (resp.status == "ok") {
			setDownloading(true, 50);
            let path = resp.result.path;
			downloadFile(path, "BeyondEClog.zip",(progress)=> {
				setDownloading(true, 50 + 50 * progress);	
			},
			() => {
				document.getElementById("dowloadLogBtn").disabled = false;
				setDownloading(false);	
			});
		}
		else {
			alert(getResources("other:LOG_DOWNLOAD_FAILED"));
			document.getElementById("dowloadLogBtn").disabled = false;
			setDownloading(false);
		}
	})
	.catch(() => {
		alert(getResources("other:LOG_DOWNLOAD_FAILED"));
		document.getElementById("dowloadLogBtn").disabled = false;
		setDownloading(false);
	})
}

/**
 * ファイルダウンロード
 * @param {string} url URL 
 * @param {string} filename 保存ファイル名
 * @param {function} progress 進捗コールバック関数
 * @param {function} finished 完了コールバック関数
 */
function downloadFile(url, filename, progress, finished) {
  
	// XMLHttpRequestオブジェクトを作成する
	var xhr = new XMLHttpRequest();
	xhr.open("GET", url, true);
	xhr.responseType = "blob"; // Blobオブジェクトとしてダウンロードする
	xhr.onload = function (oEvent) {
	  // ダウンロード完了後の処理を定義する
	  var blob = xhr.response;
		// Blobオブジェクトを指すURLオブジェクトを作る
		var objectURL = window.URL.createObjectURL(blob);
		// リンク（<a>要素）を生成し、JavaScriptからクリックする
		let a = document.createElement("a");
		a.setAttribute("href", objectURL);
		a.setAttribute("download", filename);
		a.style.display = "none";
		document.body.appendChild(a);
		a.click();
		document.body.removeChild(a);

		if (finished) {
			finished();
		}
	};
	xhr.onprogress = function(event) { 
		// event.loaded - ダウンロードされたバイト
		// event.lengthComputable = サーバが Content-Length ヘッダを送信した場合は true
		// event.total - トータルのバイト数(lengthComputable が true の場合)
		// alert(`Received ${event.loaded} of ${event.total}`);
		if (progress) {
			progress(event.loaded/event.total);
		}
	};
	// XMLHttpRequestオブジェクトの通信を開始する
	xhr.send();
}
/**
 * ダウンロード中イメージ表示
 * @param {bool} downloading - 処理中 
 * @param {number} progress - 進捗度 
 */
function setDownloading(downloading, progress) {
	if(downloading) {
		document.getElementById("downloading").style.display = "inline-block";
		document.getElementById("downloading").value = progress;
	}
	else {
		document.getElementById("downloading").style.display = "none";
	}
}

/**
 * 工場出荷状態に戻す
 */
 function restoreFactorySetting() {
	if(confirm(getResources("other:FACTORY_SETTING_CONFIRM"))) {
		if(confirm(getResources("other:FACTORY_SETTING_CONFIRM2"))) {
			document.getElementById("factorySettingBtn").disabled = true;
			doPost("/restore/factorySetting",{}, handleLoginError)
			.then((resp) => {
				if (resp.status == "ok") {
				}
				else {
					alert(getResources("other:FACTORY_SETTING_FAILED"));
				}
				document.getElementById("factorySettingBtn").disabled = false;
			})
			.catch(()=> {
				alert(getResources("other:FACTORY_SETTING_FAILED"));
				document.getElementById("factorySettingBtn").disabled = false;
			});
		}
	}
}


/**
 * SSH設定
 */
function setSsh() {
	let sshSelect = document.getElementById("sshSelect");
	let param = {
		"ssh": sshSelect.value == "1"
	}
	disableSpecialControls(true)
	let msg = "SSH";
	let enable = param.ssh? getResources("other:SSH_ENABLE"): getResources("other:SSH_DISABLE");
	doPost("/set/ssh", param, handleLoginError)
	.then((resp) => {
		if (resp.status == "ok") {
			alert(formatMessage(getResources("other:SSH_SUCCESS"), enable));
		}
		else {
			alert(formatMessage(getResources("other:SSH_FAILED"), enable));
		}
		disableSpecialControls(false)
	})
	.catch( () => {
		alert(formatMessage(getResources("other:SSH_FAILED"), enable));
		disableSpecialControls(false)
	});

}

/**
 * ログレベル設定
 */
function setLogLevel() {
	let logLevelSelect = document.getElementById("logLevelSelect");
	let param = {
		"logLevel": parseInt(logLevelSelect.value)
	}
	disableSpecialControls(true)
	doPost("/set/logLevel", param, handleLoginError)
	.then((resp) => {

		if (resp.status == "ok") {
			alert(getResources("other:LOG_LEVEL_SUCCESS"));
		}
		else {
			alert(getResources("other:LOG_LEVEL_FAILED"));
		}
		disableSpecialControls(false)
	})
	.catch( () => {
		alert(getResources("other:LOG_LEVEL_FAILED"));
		disableSpecialControls(false)
	});

}

/**
 * 特殊設定情報取得
 *
 * @returns Promise
 */
function getSpecial() {
	return doPost("/get/special", {}, handleLoginError)
	.then((resp) => {
		return new Promise((resolve, reject) => {
			if (resp.status == "ok") {
				resolve(resp.result);
			}
			else {
				reject();
			}
		});
	});
}

/**
 * 再起動
 */
function initData() {
	disableSpecialControls(true)
	doPost("/init/data",{}, handleLoginError)
	.then((resp) => {
		if (resp.status == "ok") {
			alert(getResources("other:INITDATA_SUCCESS"));
		}
		else {
			alert(getResources("other:INITDATA_FAILED"));
		}
		disableSpecialControls(false)
	})
	.catch(()=> {
		alert(getResources("other:INITDATA_FAILED"));
		disableSpecialControls(false)
	});
}

/**
 * 再起動
 */
function reboot() {
	disableSpecialControls(true)
	doPost("/reboot",{}, handleLoginError)
	.then((resp) => {
		if (resp.status == "ok") {
		}
		else {
			alert(getResources("other:REBOOT_FAILED"));
		}
		disableSpecialControls(false)
	})
	.catch(()=> {
		alert(getResources("other:REBOOT_FAILED"));
		disableSpecialControls(false)
	});
}
