window.onload = funcOnload;
window.addEventListener("beforeunload", () => {
    window.opener.focus();
});
let ssidInfo;
let ssidSetting;

/**
 * ボディオンロードイベント処理
 *
 */
function funcOnload() {
	document.getElementById('saveBtn').onclick = funcUpdate;
	document.getElementById('deleteBtn').onclick = deleteSetting;
    document.getElementById('closeBtn').onclick = () => { window.close(); };
    document.getElementById('disp_password').onchange = (e) => { 
        let wlanPassword = getDataElem("ssidSetting", "password");
        if (e.currentTarget.checked) {
            wlanPassword.type="text";
        }
        else {
            wlanPassword.type="password";
        }
    };
	addOnChangeListnerToInput("ssidSetting", funcChange);

    if (!window.opener || window.opener.closed) {
        window.close();
    }
    else {
        try {
            ssidInfo = window.opener.getSsidParam();
        }
        catch(e) {
            //test code
            ssidInfo = {
                "cellnumber": "01",
                "mac": "XX:XX:XX:XX:XX:XX",
                "frequency": "2.432",
                "frequency_units": "GHz",
                "channel": "5",
                "signal_quality": "57",
                "signal_total": "70",
                "signal_level_dBm": "-73",
                "essid": "ssid1234-a",
                "mode": "Master",
                "encryptions": [
                    {
                        "encryption": "wpa2",
                        "groupCipher": "CCMP",
                        "pairwiseCiphers": [
                            "CCMP"
                        ],
                        "authenticationSuites": [
                            "PSK"
                        ]
                    },
                    {
                        "encryption": "wpa",
                        "groupCipher": "CCMP",
                        "pairwiseCiphers": [
                            "CCMP"
                        ],
                        "authenticationSuites": [
                            "PSK"
                        ]
                    }
                ]
            };
        }

        if (ssidInfo) {
            let title = document.getElementById("h_ssid_name");
            title.textContent = ssidInfo["essid"];

            let encryption = getEncryption(ssidInfo);
            if (encryption["encType"] == "none") {
                document.getElementById("TR_wlan_password").style.display = "none";
            }

            getSsidSetting(ssidInfo["essid"])
            .then((ssid_setting) => {
                if (ssid_setting) {
                    ssidSetting = ssid_setting;
                    document.getElementById("deleteBtn").style.display = "block";
                }
                else {
                    ssidSetting = {
                        "autoIp": true,
                        "ipAddress": "",
                        "subnetMask": "",
                        "defaultGw": "",
                        "autoDNS": true,
                        "dns1": "",
                        "dns2": "",
                        "wpa_network": null
                    };
                }
                setDictionaryData("ssidSetting", ssidSetting);
                setRadioButton(ssidSetting.autoIp, ssidSetting.autoDNS);

            });
        }
        else {
            window.close();
        }
    }
}

/**
 * Radio Buttonの設定
 * @param {boolean} autoIp - 自動IPアドレス
 * @param {boolean} autoDNS - 自動DNS
 */
function setRadioButton(autoIp, autoDNS) {
	let parentId = "ssidSetting";

    let rdoAutoIp1 = getDataElem(parentId, "autoIp1");
    let rdoAutoIp2 = getDataElem(parentId, "autoIp2");

    rdoAutoIp1.onclick = () => { funcClickDHCP(false, true); };
    rdoAutoIp2.onclick = () => { funcClickDHCP(false, false); };

    if (autoIp == true) {
        rdoAutoIp1.checked = true;
        rdoAutoIp2.checked = false;
        funcClickDHCP(false, true);
    }
    else {
        rdoAutoIp1.checked = false;
        rdoAutoIp2.checked = true;
        funcClickDHCP(false, false);
    }

    let rdoAutoDNS1 = getDataElem(parentId, "autoDNS1");
    let rdoAutoDNS2 = getDataElem(parentId, "autoDNS2");
    rdoAutoDNS1.onclick = () => { funcClickDHCP(true, true); };
    rdoAutoDNS2.onclick = () => { funcClickDHCP(true, false); };
    if (autoDNS == true) {
        rdoAutoDNS1.checked = true;
        rdoAutoDNS2.checked = false;
        funcClickDHCP(true, true);
    }
    else {
        rdoAutoDNS1.checked = false;
        rdoAutoDNS2.checked = true;
        funcClickDHCP(true, false);
    }
}

/**
 * DHCP変更時処理
 * @param {boolean} dns - true:DNS、false:IP 
 * @param {boolean} flag  - true:自動、false:固定
 */
function funcClickDHCP(dns, flag) {

    let parentId = "ssidSetting";
	let ip = getDataElem(parentId, "ipAddress");
	let sub = getDataElem(parentId, "subnetMask");
	let gw = getDataElem(parentId, "defaultGw");
	let dns1 = getDataElem(parentId, "dns1");
	let dns2 = getDataElem(parentId, "dns2");

	var gray = '#808080';
	if(dns == true) {
		if(flag) {
			//背景色をグレーに
			dns1.style.backgroundColor = gray;
			dns2.style.backgroundColor = gray;
		}
		else {
			//背景色を白に
			dns1.style.backgroundColor = '';
			dns2.style.backgroundColor = '';
		}
	}
	else {
		if(flag) {
			//背景色をグレーに
			ip.style.backgroundColor = gray;
			sub.style.backgroundColor = gray;
			gw.style.backgroundColor = gray;
		}
		else {
			//背景色を白に
			ip.style.backgroundColor = '';
			sub.style.backgroundColor = '';
			gw.style.backgroundColor = '';
		}
	}

}
/**
 * 入力コントロールの無効・有効
 * @param {boolean} disabled - 無効
 */
function disableControls(disabled) {
	setInputDisabled("ssidSetting", disabled);
	document.getElementById('saveBtn').disabled = disabled;
	document.getElementById('deleteBtn').disabled = disabled;
    document.getElementById('closeBtn').disabled = disabled;
}

/**
 * 入力項目チェック
 * @returns 結果
 */
function funcInputCheck() {
	var bCheckResult = true;



	//無線LAN パスワード
	let wlanPassword = getDataElem("ssidSetting", "password");
    let encryption = getEncryption(ssidInfo);
	clearErrorOnTrById("TR_wlan_password", "P_wlan_password");
	if ( encryption["encType"] != "none" && ssidSetting["wpa_network"] == null && wlanPassword.value == "") {
		setErrorOnTrById("TR_wlan_password", "P_wlan_password", getResources("ssid_setting:PASSWORD_ERROR_REQUIRED"));
		bCheckResult = false;
	}

	//無線LAN IPアドレス
	let wlanAutoIp = getDataElem("ssidSetting", "autoIp1");
	let wlanIpAddress = getDataElem("ssidSetting", "ipAddress");
	clearErrorOnTrById("TR_wlan_ipAddress", "P_wlan_ipAddress");
	if(wlanAutoIp.checked == false) {
		if(wlanIpAddress.value == '') {
			setErrorOnTrById("TR_wlan_ipAddress", "P_wlan_ipAddress", getResources("ssid_setting:IPADDR_ERROR_REQUIRED"));
			bCheckResult = false;
		}
		else if (funcCheckIpAddress(wlanIpAddress.value) == false) {
			setErrorOnTrById("TR_wlan_ipAddress", "P_wlan_ipAddress", getResources("ssid_setting:IPADDR_ERROR_FORMAT"));
			bCheckResult = false;
		}
	}

	//無線LAN サブネットマスク
	let wlanSubnetMask = getDataElem("ssidSetting", "subnetMask");
	clearErrorOnTrById("TR_wlan_subnetMask", "P_wlan_subnetMask");
	if(wlanAutoIp.checked == false) {
		if(wlanSubnetMask.value == '') {
			setErrorOnTrById("TR_wlan_subnetMask", "P_wlan_subnetMask", getResources("ssid_setting:SUBNET_ERROR_REQUIRED"));
			bCheckResult = false;
		}
		else if (funcCheckIpAddress(wlanSubnetMask.value) == false) {
			setErrorOnTrById("TR_wlan_subnetMask", "P_wlan_subnetMask", getResources("ssid_setting:SUBNET_ERROR_FORMAT"));
			bCheckResult = false;
		}
	}

	//無線LAN デフォルトGW
	let wlanDefaultGw = getDataElem("ssidSetting", "defaultGw");
	clearErrorOnTrById("TR_wlan_defaultGw", "P_wlan_defaultGw");
	if(wlanAutoIp.checked == false) {
		if (wlanDefaultGw.value != "" && funcCheckIpAddress(wlanDefaultGw.value) == false) {
			setErrorOnTrById("TR_wlan_defaultGw", "P_wlan_defaultGw", getResources("ssid_setting:GW_ERROR_FORMAT"));
			bCheckResult = false;
		}
	}


	//無線LAN デフォルトDNS
	let wlanAutoDNS = getDataElem("ssidSetting", "autoDNS1");
	let wlanDNS1 = getDataElem("ssidSetting", "dns1");
	clearErrorOnTrById("TR_wlan_dns1", "P_wlan_dns1");
	if (wlanAutoDNS.checked == false) {
		if (wlanDNS1.value != "" && funcCheckIpAddress(wlanDNS1.value) == false) {
			setErrorOnTrById("TR_wlan_dns1", "P_wlan_dns1", getResources("ssid_setting:DNS1_ERROR_FORMAT"));
			bCheckResult = false;
		}
	}
	let wlanDNS2 = getDataElem("ssidSetting", "dns2");
	clearErrorOnTrById("TR_wlan_dns2", "P_wlan_dns2");
	if (wlanAutoDNS.checked == false) {
		if (wlanDNS2.value != "" && funcCheckIpAddress(wlanDNS2.value) == false) {
			setErrorOnTrById("TR_wlan_dns2", "P_wlan_dns2", getResources("ssid_setting:DNS2_ERROR_FORMAT"));
			bCheckResult = false;
		}
	}

	if(bCheckResult == false) {
		showDivError();
		return false;
	} else {
		hideDivError();
		return true;
	}
}


/**
 * SSID設定情報の取得
 * @param {string} ssid - ssid
 */
function getSsidSetting(ssid) {
    let param = {
        "ssid": ssid
    };
	return doPost("/get/ssidSetting", param, handleLoginError)
	.then((resp) => {
		return new Promise((resolve, reject) => {
			if (resp.status == "ok") {
				resolve(resp.result);
			}
			else {
				reject()
			}
		});
	});
	
}

/**
 * 更新
 */
function doUpdate() {
    saveSetting();
}

/**
 * 設定の保存
 */
function saveSetting() {
    disableControls(true);
    let param = {
        "ssid": ssidInfo["essid"]
    };

	let ssidSetting = getDictionaryData("ssidSetting");
	let autoIp = getDataElem("ssidSetting", "autoIp1");
	let autoDNS = getDataElem("ssidSetting", "autoDNS1");
	ssidSetting["autoIp"] = autoIp.checked;
	ssidSetting["autoDNS"] = autoDNS.checked;
	delete ssidSetting["autoIp1"];
	delete ssidSetting["autoIp2"];
	delete ssidSetting["autoDNS1"];
	delete ssidSetting["autoDNS2"];

    let encryption = getEncryption(ssidInfo);
    ssidSetting["encryption"] = encryption;
    param["ssidSetting"] = ssidSetting;

	doPost("/save/ssidSetting", param, handleLoginError)
	.then((resp) => {
		if (resp.status == "ok") {
            saveFinished();
		}
		else {
            showServerError(getResources("ssid_setting:UPDATE_FAILED"));
		}
        disableControls(false);
	})
	.catch( () => {
        showServerError(getResources("ssid_setting:UPDATE_FAILED"));
        disableControls(false);
	});

}

/**
 * 設定の削除
 */
function deleteSetting() {
    let param = {
        "ssid": ssidInfo["essid"]
    };
    disableControls(true);
	doPost("/delete/ssidSetting", param, handleLoginError)
	.then((resp) => {
		if (resp.status == "ok") {
            deleteFinished();
		}
		else {
            showServerError(getResources("ssid_setting:DELETE_FAILED"));
		}
        disableControls(false);
	})
	.catch( () => {
        showServerError(getResources("ssid_setting:DELETE_FAILED"));
        disableControls(false);
	});

}
/**
 * SSIDの暗号方式の取得
 * @param {object} ssidInfo - SSID情報
 * @returns 暗号方式
 */
function getEncryption(ssidInfo) {
	let encryption = {"encType" : "none"};
    if (ssidInfo.encryptions.length > 0) {
        let wpa2Arr = ssidInfo.encryptions.filter((e) => e.encryption == "wpa2");
        let wpaArr = ssidInfo.encryptions.filter((e) => e.encryption == "wpa");
        let wepArr = ssidInfo.encryptions.filter((e) => e.encryption == "wep");
        if (wpa2Arr.length > 0) {
            cipher = "CCMP";
            if (wpa2Arr[0].groupCipher) {
                cipher = wpa2Arr[0].groupCipher;
            }
            encryption = {
                "encType" : "wpa2",
                "cipher": cipher
            };
        }
        else if (wpaArr.length > 0) {
            cipher = "TKIP";
            if (wpaArr[0].groupCipher) {
                cipher = wpaArr[0].groupCipher;
            }
            encryption = {
                "encType" : "wpa",
                "cipher": cipher
            };
        }
        else if (wepArr.length > 0) {
            encryption = {
                "encType" : "wep"
            };
        }
    }

	return encryption;
}

/**
 * 保存終了
 */
function saveFinished() {
    window.close();
    if (window.opener && window.opener.saveSsidSettingFinished) {
        // 親画面の関数を呼ぶ
        window.opener.saveSsidSettingFinished(ssidInfo["essid"]);
    }
}

/**
 * 削除終了
 */
function deleteFinished() {
    window.close();
    if (window.opener && window.opener.deleteSsidSettingFinished) {
        // 親画面の関数を呼ぶ
        window.opener.deleteSsidSettingFinished(ssidInfo["essid"]);
    }
}


