import json
import pathlib
import traceback
import common_util as util

if util.is_receiver_device():
    from recv_dev.config import TerminalInfo
else:
    from disp_dev.config_setting import ConfigSetting

class Resources:
    """リソースクラス
    """
    RESOURCE_FILE = pathlib.Path("./resources.json").resolve()
    DEFAULT_LANG = "en"

    def __init__(self):
        """コンストラクタ
        """
        self.dict = None
        self.load()

    def load(self):
        """ロード
        """
        with open(self.RESOURCE_FILE) as f:
            self.dict = json.loads(f.read())

    def getLanguage(self):
        """言語取得
        
        Returns:
            string: 言語
        """
        if util.is_receiver_device():
            return TerminalInfo().language
        else:
            return ConfigSetting().language

    def getResource(self, resourceId, lang = None):
        """リソース取得
        
        Args:
            resourceId (string): リソースID
            lang (string, optional): 言語. Defaults to None.
        
        Returns:
            string: 文字列
        """
        if lang is None or len(lang) == 0:
            lang = self.getLanguage()

        if not lang in self.dict:
            lang = util.get_system_language()

            if not lang in self.dict:
                lang = self.DEFAULT_LANG

        try:
            return self.dict[lang][resourceId]
        except:
            traceback.print_exc()
            return ""


        # if resourceId in self.dict[lang]:
        #     return self.dict[lang][resourceId]
        # else:
        #     return ""
