#!/bin/bash
ECOMONI_DIR='/home/pi/ecomoni/'
DOWNLOAD_DIR='/home/pi/temp/firmware/'
DL_ECOMONI_DIR='/home/pi/temp/firmware/ecomoni/'
ZIP_FILE='*.zip'
PWD=$(pwd)
LOG_FILE="${ECOMONI_DIR}app/log/update.log"

mkfifo /tmp/update_message

function log() {
    msg=$1
    echo "$(date +'%Y/%m/%d %T'), ${msg}" | tee -a ${LOG_FILE}
}

function progress() {
    p=$1
    echo "$p" | tee /tmp/update_message 
}

log 'Update start.'
sleep 1
if [ -d ${DOWNLOAD_DIR} ]; then
    cd ${DOWNLOAD_DIR}
    ls ${ZIP_FILE} >/dev/null 2>&1
    if [  $? -eq 0 ]; then
        export DISPLAY=:0.0
        /usr/bin/python3 ${ECOMONI_DIR}script/update-disp.py &
        UPDATE_PID=$!
        progress 0

        for file in ${ZIP_FILE}; do
            log "zipfile = ${file}"
            unzip $file
        done
        progress 2
        
        if [ -d ${DL_ECOMONI_DIR} ]; then
            sleep 2
            progress 3
            BACKUP_DIR="${DL_ECOMONI_DIR}bk/"
            mkdir -p ${BACKUP_DIR}
            cp -pr ${ECOMONI_DIR}* ${BACKUP_DIR} 
            progress 5

            rsync -avI ${DL_ECOMONI_DIR} ${ECOMONI_DIR} --exclude "app/data.db" --exclude "app/recv_dev/config/" --exclude "bk/" 
            progress 7

            /usr/bin/python3 ${DL_ECOMONI_DIR}script/update.py 
            if [ $? -eq 0 ]; then 
                progress 10
                log 'Update Success.'
                rm -f ${ZIP_FILE}
                sudo systemctl daemon-reload
                sleep 2
            else
                progress -1
                log 'Update Failed.'
                cp -pr ${BACKUP_DIR}* ${ECOMONI_DIR}
                sleep 5
            fi
            rm -r ${DL_ECOMONI_DIR}
        else
            log 'No Update File.'
            rm -rf ${DOWNLOAD_DIR}*
        fi
        kill -9 ${UPDATE_PID}
    else
        log 'No Update File.'
    fi

fi
rm /tmp/update_message

cd ${PWD}